GraphDataWindow = class(Turbine.UI.Window)
function GraphDataWindow:Constructor()
    self.width = 1000
    self.height = 600
    self.x = Turbine.UI.Display.GetWidth() / 2 - self.width / 2
    self.y = Turbine.UI.Display.GetHeight() / 2 - self.height / 2

	self.borderSize = 2

	self.borderWidth = 1
	self.titleHeight = 0
	self.itemWidth = 1
	self.dotWidth = 5
	self.lineWidth = 2
	self.itemColour = Turbine.UI.Color.Black
	self.dotColour = Turbine.UI.Color.White
	self.dotSelectedColour = Turbine.UI.Color.Red
	self.lineColour = Turbine.UI.Color.White
	self.graphItemWidth = 0

	Turbine.UI.Window.Constructor(self)
    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    self:SetText("Data")
    self:SetVisible(false)
    self:SetBackColor(Turbine.UI.Color.White)

	self.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end
  
    self.MouseUp = function()
        self.Moving = false
    end
  
    self.MouseMove = function(sender,args)
        if self.Moving then
          local newLeft = self:GetLeft() - (self.MoveX - args.X)
          local newTop = self:GetTop() - (self.MoveY - args.Y)
          self:SetPosition(newLeft,newTop)
        end
    end

	self.innerBackground = Turbine.UI.Control()
	self.innerBackground:SetParent(self)
	self.innerBackground:SetPosition(self.borderSize, self.borderSize)
	self.innerBackground:SetSize(self.width - 2 * self.borderSize, self.height - 2 * self.borderSize)
	self.innerBackground:SetBackColor(Turbine.UI.Color.Black)
	self.innerBackground:SetMouseVisible(false)

	self.viewingControl = Turbine.UI.Control()
	self.viewingControl:SetParent(self.innerBackground)
	self.viewingControl:SetBackColor(Turbine.UI.Color.White)
	self.viewingControl:SetSize(300, 22)
	self.viewingControl:SetMouseVisible(false)

	self.closeControl = Turbine.UI.Control()
	self.closeControl:SetParent(self)
	self.closeControl:SetBackColor(Turbine.UI.Color(1,1,0,0))
	self.closeControl:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
	self.closeControl:SetSize(10, 10)
	self.closeControl.MouseClick = function()
		self:SetVisible(false)
	end

	self.viewingLabel = Turbine.UI.Label()
	self.viewingLabel:SetParent(self.viewingControl)
	self.viewingLabel:SetBackColor(Turbine.UI.Color.Black)
	self.viewingLabel:SetSize(300 - 2, 20)
	self.viewingLabel:SetPosition(1, 1)
    self.viewingLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.viewingLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold22)
    self.viewingLabel:SetOutlineColor(Turbine.UI.Color.Black)
    self.viewingLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.viewingLabel:SetForeColor(Turbine.UI.Color(0.73,0.68,0.36))
    self.viewingLabel:SetText("Graph Categories")
	self.viewingLabel:SetMouseVisible(false)

	self.viewingListboxControl = Turbine.UI.Control()
	self.viewingListboxControl:SetParent(self.innerBackground)
	self.viewingListboxControl:SetBackColor(Turbine.UI.Color.White)
	self.viewingListboxControl:SetSize(300, self.innerBackground:GetHeight() - 22)
	self.viewingListboxControl:SetPosition(0, 22)
	self.viewingListboxControl:SetMouseVisible(false)


	self.viewingListbox = Turbine.UI.ListBox()
	self.viewingListbox:SetParent(self.innerBackground)
	self.viewingListbox:SetSize(300 - 2, self.innerBackground:GetHeight() - 22)
	self.viewingListbox:SetPosition(1, 22)
	self.viewingListbox:SetBackColor(Turbine.UI.Color.Black)

	self.graphBackground = Turbine.UI.Control()
	self.graphBackground:SetParent(self.innerBackground)
	self.graphBackground:SetSize(self.innerBackground:GetWidth() - self.viewingListboxControl:GetWidth(), self.innerBackground:GetHeight())
	self.graphBackground:SetPosition(self.viewingListboxControl:GetWidth(), 0)

	self.graphScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.graphScrollbar:SetParent(self.graphBackground)
    self.graphScrollbar:SetPosition(0, self.graphBackground:GetHeight() - 10)
    self.graphScrollbar:SetSize(self.graphBackground:GetWidth(), 10)
    --self.graphScrollbar:SetBackColor(Turbine.UI.Color.Red)
	self.graphScrollbar:SetOrientation(Turbine.UI.Orientation.Horizontal)
	self.graphScrollbar:SetVisible(false)

	self.listboxCheckboxOptions = {
		["DPS"] = "DPSMapped",
		["Damage"] = "DamageMapped",
		["HPS (Received)"] = "HealingReceivedMapped",
		["HPS (Applied)"] = "HealingAppliedMapped",
		["TPS"] = "TPSMapped",
		["Taken"] = "DamageTakenMapped",
		["Morale"] = "MoraleMapped",
		["Power"] = "PowerMapped",
	}

	self.listboxNames = {
		["DamageMapped"] = "Damage",
		["DPSMapped"] = "DPS",
		["HealingReceivedMapped"] = "HPS (Received)",
		["HealingAppliedMapped"] = "HPS (Applied)",
		["DamageTakenMapped"] = "Taken",
		["TPSMapped"] = "TPS",
		["MoraleMapped"] = "Morale",
		["PowerMapped"] = "Power",
		["WrathMapped"] = "Wrath",
	}

	self.graphLabels = {
		["DamageMapped"] = {"Time", "Damage"},
		["DPSMapped"] = {"Time", "DPS"},
		["HealingReceivedMapped"] = {"Time", "Healing"},
		["HealingAppliedMapped"] = {"Time", "Healing"},
		["DamageTakenMapped"] = {"Time", "Damage"},
		["TPSMapped"] = {"Time", "TPS"},
		["MoraleMapped"] = {"Time", "Morale"},
		["PowerMapped"] = {"Time", "Power"},
		["WrathMapped"] = {"Time", "Wrath"},
	}

	self.graphUnits = {
		["DamageMapped"] = {"s", ""},
		["DPSMapped"] = {"s", ""},
		["HealingReceivedMapped"] = {"s", "Received"},
		["HealingAppliedMapped"] = {"s", "Applied"},
		["DamageTakenMapped"] = {"s", "Taken"},
		["TPSMapped"] = {"s", ""},
		["MoraleMapped"] = {"s", ""},
		["PowerMapped"] = {"s", ""},
		["WrathMapped"] = {"s", ""},
	}

	self.graphColours = {
		["DamageMapped"] = Turbine.UI.Color.DeepPink,
		["DPSMapped"] = Turbine.UI.Color.Red,
		["HealingReceivedMapped"] = Turbine.UI.Color.LightSlateGray,
		["HealingAppliedMapped"] = Turbine.UI.Color.Teal,
		["DamageTakenMapped"] = Turbine.UI.Color.Gray,
		["TPSMapped"] = Turbine.UI.Color.LightSlateGray,
		["MoraleMapped"] = Turbine.UI.Color.DarkGreen,
		["PowerMapped"] = Turbine.UI.Color.Blue,
		["WrathMapped"] = Turbine.UI.Color.OrangeRed,
	}

	if Turbine.Gameplay.LocalPlayer.GetInstance():GetClass() == Turbine.Gameplay.Class.Beorning then
		self.listboxCheckboxOptions["Wrath"] = "WrathMapped"
	end

	--self.effectTreeview = Turbine.UI.TreeView()

	self.ActiveGraphs = {}

	self.checkboxes = {}

	self.checkCount = 0

	for key, value in spairs(self.listboxCheckboxOptions, function(t, a, b) return a < b end) do
		self.checkboxes[key] = ListBoxCheckBox(self.viewingListbox:GetWidth(), 20, key, 
		function(checked)
			--[[
			if checked then
				for k, v in pairs(self.checkboxes) do
					if k ~= key then
						v.checkBox:SetChecked(false)
					end 
				end
			end
			--]]
			if checked then
				self.checkCount = self.checkCount + 1
				self.checkboxes[key]:SetForeColor(self.graphColours[self.listboxCheckboxOptions[key]])
			else
				self.checkCount = self.checkCount - 1
				self.checkboxes[key]:SetForeColor(Turbine.UI.Color.White)
			end
			self:SetGraphType(value, checked)
		end)
		self.viewingListbox:AddItem(self.checkboxes[key])
	end

	self.viewingListbox:SetHeight(self.viewingListbox:GetItemCount() * 20)

	self.effectListbox = Turbine.UI.ListBox()
	self.effectListbox:SetParent(self.viewingListboxControl)
	self.effectListbox:SetPosition(0, self.viewingListbox:GetHeight())
	self.effectListbox:SetSize(self.viewingListbox:GetWidth() - 10, self.viewingListboxControl:GetHeight() - self.viewingListbox:GetHeight())
	self.effectListbox:SetBackColor(Turbine.UI.Color.Black)

	self.effectListboxScrollbarControl = Turbine.UI.Control()
	self.effectListboxScrollbarControl:SetParent(self.viewingListboxControl)
	self.effectListboxScrollbarControl:SetPosition(self.effectListbox:GetWidth(), self.viewingListbox:GetHeight())
	self.effectListboxScrollbarControl:SetSize(10, self.viewingListboxControl:GetHeight() - self.viewingListbox:GetHeight())
	self.effectListboxScrollbarControl:SetBackColor(Turbine.UI.Color.Black)

	self.effectListboxScrollbar = Turbine.UI.Lotro.ScrollBar()
	self.effectListboxScrollbar:SetParent(self.viewingListboxControl)
	self.effectListboxScrollbar:SetPosition(self.effectListbox:GetWidth(), self.viewingListbox:GetHeight())
	self.effectListboxScrollbar:SetSize(10, self.viewingListboxControl:GetHeight() - self.viewingListbox:GetHeight())
	self.effectListboxScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
	self.effectListboxScrollbar:SetBackColor(Turbine.UI.Color.Black)

	self.effectListbox:SetVerticalScrollBar(self.effectListboxScrollbar)

	self.effectIconListbox = Turbine.UI.ListBox()
	self.effectIconListbox:SetParent(self.graphBackground)
	self.effectIconListbox:SetSize(20, self.graphBackground:GetHeight())
	self.effectIconListbox:SetMouseVisible(false)
	--self.effectIconListbox:SetBackColor(Turbine.UI.Color.Red)
	self.effectIconListbox:SetZOrder(100)

	self.effectBarListboxOfListBoxes = Turbine.UI.ListBox()
	self.effectBarListboxOfListBoxes:SetParent(self.graphBackground)
	self.effectBarListboxOfListBoxes:SetSize(self.graphBackground:GetWidth() - 20, self.graphBackground:GetHeight())
	self.effectBarListboxOfListBoxes:SetPosition(20, 0)
	self.effectBarListboxOfListBoxes:SetMouseVisible(false)
	--self.effectBarListboxOfListBoxes:SetBackColor(Turbine.UI.Color.Blue)
	--self.effectIconListbox:SetZOrder(1)

	self.barListboxes = {}
end

function GraphDataWindow:SetEncounterData(data)
	self.encounterData = data

	self.effectListbox:ClearItems()
	self.effectIconListbox:ClearItems()
	self.effectBarListboxOfListBoxes:ClearItems()
	for key, value in pairs(self.checkboxes) do
		value:SetChecked(false)
	end
	for key, value in spairs(self.encounterData.EffectApplicationTimes, function(t, a, b) return a < b end) do
		self.effectListbox:AddItem(self:CreateEffectNode(key))
	end


	if not self:IsVisible() then self:SetVisible(true) end

	for key, value in pairs(self.ActiveGraphs) do
		self.ActiveGraphs[key]:Unload()
		self.ActiveGraphs[key] = nil
	end

	self.ActiveGraphs = {}
end

function GraphDataWindow:CreateEffectNode(info)
	local name, icon = string.match(info, "(.*)@@(.*)")

	local item = Turbine.UI.Control()
	item:SetSize(self.viewingListbox:GetWidth() - 10, 32 + 2)
	item:SetBackColor(Turbine.UI.Color.White)
	
	item.icon = Turbine.UI.Control()
	item.icon:SetParent(item)
	item.icon:SetSize(32, 32)
	item.icon:SetPosition(1, 1)
	item.icon:SetBackground(tonumber(icon))
	item.icon:SetMouseVisible(false)

	item.label = Turbine.UI.Label()
	item.label:SetParent(item)
	item.label:SetSize(item:GetWidth() - 32, 32)
	item.label:SetPosition(33, 1)
	item.label:SetText(name)
	item.label:SetMouseVisible(false)
	item.label:SetBackColor(Turbine.UI.Color.Black)
	item.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
	item.label:SetOutlineColor(Turbine.UI.Color.Black)
	item.label:SetFontStyle(Turbine.UI.FontStyle.Outline)

	item.selected = false

	item.MouseEnter = function()
		if not item.selected then
			item.label:SetBackColor(Turbine.UI.Color.DarkGray)
		end
	end

	item.MouseLeave = function()
		if not item.selected then
			item.label:SetBackColor(Turbine.UI.Color.Black)
		end
	end

	item.MouseClick = function()
		if self.checkCount > 0 then
			item.selected = not item.selected
			if item.selected then
				item.label:SetBackColor(Turbine.UI.Color.Teal)

				
				item.createdItem = self:CreateEffectIcon(icon)
				self:AddBarListbox(item.createdItem)
				self:FillBarListbox(self.barListboxes[item.createdItem], info)
				self.effectIconListbox:AddItem(item.createdItem)


			else
				if item.createdItem then
					self.effectBarListboxOfListBoxes:RemoveItem(self.barListboxes[item.createdItem])
					self.effectIconListbox:RemoveItem(item.createdItem)
					item.createdItem:SetVisible(false)
					item.createdItem.icon:SetVisible(false)
					item.createdItem.icon:SetParent(nil)
					item.createdItem = nil
				end
				item.label:SetBackColor(Turbine.UI.Color.DarkGray)
			end
		end
	end

	return item
end

function GraphDataWindow:AddBarListbox(item)
	local listbox = Turbine.UI.ListBox()
	listbox:SetSize(self.graphBackground:GetWidth() - 20, 20)
	--listbox:SetBackColor(Turbine.UI.Color.Purple)
	listbox:SetOrientation(Turbine.UI.Orientation.Horizontal)
	self.graphScrollbar:SetVisible(true)
	listbox:SetHorizontalScrollBar( self.graphScrollbar)

	self.barListboxes[item] = listbox

	self.effectBarListboxOfListBoxes:AddItem(listbox)
end

function GraphDataWindow:FillBarListbox(listbox, key)
	if self.encounterData.EffectApplicationTimes[key][1] ~= 0 then
		listbox:AddItem(self:CreateBarControl(self.encounterData.EffectApplicationTimes[key][1], false))
	end

	local lastItem = 0
	for index, value in ipairs(self.encounterData.EffectApplicationTimes[key]) do
		if index > 1 then
			listbox:AddItem(self:CreateBarControl(value - self.encounterData.EffectRemovalTimes[key][index - 1], false))
		end

		local endTime = self.encounterData.EffectRemovalTimes[key] and self.encounterData.EffectRemovalTimes[key][index] or 
						self.encounterData.CombatEndTime and self.encounterData.CombatEndTime - self.encounterData.CombatStartTime or
						self.encounterData.currentGameTime and self.encounterData.currentGameTime - self.encounterData.CombatStartTime
		listbox:AddItem(self:CreateBarControl(endTime - value, true))

		lastItem = endTime
	end

	local combatEnd = (self.encounterData.CombatEndTime and self.encounterData.CombatEndTime - self.encounterData.CombatStartTime or
	self.encounterData.currentGameTime and self.encounterData.currentGameTime - self.encounterData.CombatStartTime) 
	if lastItem < combatEnd then
		listbox:AddItem(self:CreateBarControl(combatEnd - lastItem, false))
	end
end

function GraphDataWindow:CreateBarControl(seconds, fill)
	local item = Turbine.UI.Control()

	item:SetSize(seconds * self.graphItemWidth, 20)
	if fill then
		item.applicationBar = Turbine.UI.Control()
		item.applicationBar:SetParent(item)
		item.applicationBar:SetSize(math.min(3, item:GetWidth()), 10)
		item.applicationBar:SetPosition(0, 5)
		item.applicationBar:SetBackColor(Turbine.UI.Color(0.73,0.68,0.36))
	
		item.timeBar = Turbine.UI.Control()
		item.timeBar:SetParent(item)
		item.timeBar:SetPosition(item.applicationBar:GetWidth(), 8)
		item.timeBar:SetSize(item:GetWidth() - item.applicationBar:GetWidth(), 4)
		item.timeBar:SetBackColor(Turbine.UI.Color(0.73,0.68,0.36))
	end
	return item
end

function GraphDataWindow:CreateEffectIcon(icon)
	local item = Turbine.UI.Control()
	item:SetSize(20, 20)

	item.icon = Turbine.UI.Window()
	item.icon:SetParent(item)
	item.icon:SetVisible(true)
	item.icon:SetSize(32, 32)
	item.icon:SetBackground(tonumber(icon))
	item.icon:SetStretchMode(1)
	item.icon:SetSize(20, 20)

	return item
end

function GraphDataWindow:SetGraphType(name, remove)
	if not self.encounterData then 
		--self.ActiveGraphs[name] = PrimePlugins.RaidTools.UI.GraphWindow(self.graphBackground, nil, 0, 0, "Test Graph", self.graphBackground:GetWidth(), self.graphBackground:GetHeight(), self.borderWidth, self.titleHeight, 20, self.dotWidth, self.lineWidth, self.itemColour, self.dotColour, self.dotSelectedColour, self.lineColour)
		--self.ActiveGraphs[name]:SetLabels("Time", "Damage")
		--self.ActiveGraphs[name]:SetUnits("s", "")
		return 
	end

	if self.ActiveGraphs[name] then
		if remove then
			self.ActiveGraphs[name]:SetVisible(true)
		else
			--self.ActiveGraphs[name]:SetVisible(false)
			self.ActiveGraphs[name]:Unload()
			self.ActiveGraphs[name] = nil
		end
	else
		self.graphItemWidth = math.min(math.max(math.ceil(self.graphBackground:GetWidth()/Tablelength(self.encounterData[name])), 1), 20)
		--GraphWindow(, data, x, y, title, width, height, borderWidth, titleHeight, itemWidth, dotWidth, lineWidth, itemColour, dotColour, dotSelectedColour, lineColour)
		self.ActiveGraphs[name] = self:MakeGraph(self, self.encounterData[name], self.graphBackground:GetLeft(), self.graphBackground:GetTop(), self.listboxNames[name], self.graphBackground:GetWidth(), self.graphBackground:GetHeight(), self.borderWidth, self.titleHeight, self.graphItemWidth, self.dotWidth, self.lineWidth, self.itemColour, self.dotColour, self.dotSelectedColour, self.graphColours[name])
		--ChatOutput(self.scrollbar)
		self.ActiveGraphs[name]:SetXOffset(-1)
		self.ActiveGraphs[name]:SetLabels(self.graphLabels[name][1], self.graphLabels[name][2])
		self.ActiveGraphs[name]:SetUnits(self.graphUnits[name][1], self.graphUnits[name][2])
	end
end

function GraphDataWindow:MakeGraph(parent, data, x, y, title, width, height, borderWidth, titleHeight, itemWidth, dotWidth, lineWidth, itemColour, dotColour, dotSelectedColour, lineColour)
	local graphWindow = Turbine.UI.Control()
	
    -- Numerical values
    graphWindow.width = width or 800
    graphWindow.height = height or 320
    graphWindow.outerBorder = borderWidth or 2
    graphWindow.scrollbarHeight = 10
    graphWindow.titleHeight = titleHeight or 20
    graphWindow.listBoxHeight = graphWindow.height - 2 * graphWindow.outerBorder - graphWindow.scrollbarHeight - graphWindow.titleHeight
    graphWindow.itemWidth = itemWidth or 10
    graphWindow.lowestValue = 0
    graphWindow.highestValue = 10
    graphWindow.dotWidth = dotWidth or 5
    graphWindow.lineWidth = lineWidth or 2
    graphWindow.itemWidthAdjusted = graphWindow.itemWidth - graphWindow.dotWidth / 2
    graphWindow.itemWidthSquared = math.pow(graphWindow.itemWidthAdjusted, 2)
    graphWindow.listBoxWidth = graphWindow.width - 2 * graphWindow.outerBorder
    graphWindow.minItem = 0
    graphWindow.maxItem = math.floor(graphWindow.listBoxWidth / graphWindow.itemWidth)
    graphWindow.infoWindow = GraphInfoWindow()
    graphWindow.data = data or {}
    graphWindow.xLabel = "X"
    graphWindow.yLabel = "Y"
    graphWindow.xUnit = ""
    graphWindow.yUnit = ""
    graphWindow.xOffset = 0

    -- Colour values
    graphWindow.itemColour = itemColour or Turbine.UI.Color(1, 0, 0.5, 0.6)
    graphWindow.dotColour = dotColour or Turbine.UI.Color.NavajoWhite
    graphWindow.dotSelectedColour = dotSelectedColour or Turbine.UI.Color.DarkRed
    graphWindow.lineColour = lineColour or Turbine.UI.Color.White

    if parent then
        graphWindow:SetParent(parent)
    end
    graphWindow:SetVisible(true)
    graphWindow:SetPosition(x or 500, y or 500)
    graphWindow:SetSize(graphWindow.width, graphWindow.height)
    graphWindow:SetMouseVisible(false)
    --graphWindow:SetBackColor(Turbine.UI.Color.Black)

    graphWindow.titleBar = Turbine.UI.Label()
    graphWindow.titleBar:SetParent(graphWindow)
    graphWindow.titleBar:SetVisible(true)
    graphWindow.titleBar:SetSize(graphWindow.width - 2 * graphWindow.outerBorder, graphWindow.titleHeight)
    graphWindow.titleBar:SetBackColor(Turbine.UI.Color.Black)
    graphWindow.titleBar:SetPosition(graphWindow.outerBorder, graphWindow.outerBorder)
    graphWindow.titleBar:SetMouseVisible(false)
    graphWindow.titleBar:SetFontStyle(Turbine.UI.FontStyle.Outline)
    graphWindow.titleBar:SetOutlineColor(Turbine.UI.Color.Black)
    graphWindow.titleBar:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    graphWindow.titleBar:SetForeColor(Turbine.UI.Color.White)
    graphWindow.titleBar:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    graphWindow.titleBar:SetText(title or "Graph title")
    graphWindow.titleBar:SetMouseVisible(false)


    graphWindow.graph = Turbine.UI.ListBox()
    graphWindow.graph:SetParent(graphWindow)
    graphWindow.graph:SetSize(graphWindow.listBoxWidth, graphWindow.listBoxHeight)
    graphWindow.graph:SetPosition(graphWindow.outerBorder, graphWindow.titleHeight + graphWindow.outerBorder)
    --graphWindow.graph:SetBackColor(graphWindow.itemColour)
    graphWindow.graph:SetOrientation(Turbine.UI.Orientation.Horizontal)
    graphWindow.graph:SetMouseVisible(false)

    graphWindow.scrollbar = Turbine.UI.Lotro.ScrollBar()
    graphWindow.scrollbar:SetParent(graphWindow)
    graphWindow.scrollbar:SetPosition(graphWindow.outerBorder, graphWindow.listBoxHeight + graphWindow.outerBorder + graphWindow.titleHeight)
    graphWindow.scrollbar:SetSize(graphWindow.listBoxWidth, graphWindow.scrollbarHeight)
    graphWindow.scrollbar:SetBackColor(Turbine.UI.Color.Black)
    graphWindow.scrollbar:SetVisible(false)
	self.graphScrollbar:SetVisible(true)
    graphWindow.graph:SetHorizontalScrollBar(self.graphScrollbar)

	graphWindow.ShowInfoGraph = function(sender, x)
		local xUnit = graphWindow.xUnit ~= "" and string.format(" (%s)", graphWindow.xUnit) or ""
		local yUnit = graphWindow.yUnit ~= "" and string.format(" (%s)", graphWindow.yUnit) or ""
		graphWindow.infoWindow:View(string.format("%s: %.1f%s\n%s: %.1f%s", graphWindow.xLabel, tostring((tonumber(x) or 0) + graphWindow.xOffset), xUnit, graphWindow.yLabel, graphWindow.data[tonumber(x)] and tostring(graphWindow.data[tonumber(x)][2]), yUnit))
	end
	
	graphWindow.SetScrollbar = function(scrollbar)
		graphWindow.scrollbar:SetVisible(false)
		--graphWindow.graph:SetHorizontalScrollBar(nil)
		graphWindow.graph:SetHorizontalScrollBar(scrollbar)
	end
	
	graphWindow.SetLabels = function(sender, x, y)
		graphWindow:SetXLabel(x)
		graphWindow:SetYLabel(y)
	end
	
	graphWindow.SetXLabel = function(sender, x)
		graphWindow.xLabel = tostring(x)
	end
	
	graphWindow.SetYLabel = function(sender, y)
		graphWindow.yLabel = tostring(y)
	end
	
	graphWindow.SetUnits = function(sender, x, y)
		graphWindow:SetXUnit(x)
		graphWindow:SetYUnit(y)
	end
	
	graphWindow.SetXUnit = function(sender, x)
		graphWindow.xUnit = tostring(x)
	end
	
	graphWindow.SetYUnit = function(sender, y)
		graphWindow.yUnit = tostring(y)
	end
	
	graphWindow.SetXOffset = function(sender, value)
		graphWindow.xOffset = value
	end
	
	graphWindow.AddPositionItem = function()
		local listBoxItem = Turbine.UI.Window()
		listBoxItem:SetVisible(true)
		listBoxItem:SetSize(20, graphWindow.listBoxHeight)
		listBoxItem:SetBackColor(graphWindow.itemColour)
		listBoxItem:SetMouseVisible(false)
		listBoxItem:SetVisible(false)
		listBoxItem.PositionChanged = function()
			graphWindow.minItem = math.floor(math.abs((listBoxItem:GetLeft() - graphWindow.itemWidth)/graphWindow.itemWidth)) - 1
			graphWindow.maxItem = math.floor(math.abs((listBoxItem:GetLeft() - graphWindow.listBoxWidth)/graphWindow.itemWidth))
		end
		graphWindow.graph:AddItem(listBoxItem)
	end
	
	graphWindow.GetEndItem = function()
		local listBoxItem = Turbine.UI.Control()
		listBoxItem:SetVisible(true)
		listBoxItem:SetSize(graphWindow.itemWidth, graphWindow.listBoxHeight)
		--listBoxItem:SetBackColor(graphWindow.itemColour)
		listBoxItem:SetMouseVisible(false)
		listBoxItem:SetVisible(true)
	
		local dotOneY = (1 - (graphWindow.data[graphWindow.dataCount] and graphWindow.data[graphWindow.dataCount][2] or 0) / (graphWindow.highestValue - graphWindow.lowestValue)) * (graphWindow.listBoxHeight - graphWindow.dotWidth)
	
		listBoxItem.dotOne = Turbine.UI.Control()
		listBoxItem.dotOne:SetParent(listBoxItem)
		listBoxItem.dotOne:SetSize(graphWindow.dotWidth, graphWindow.dotWidth)
		listBoxItem.dotOne:SetBackColor(graphWindow.dotColour)
		listBoxItem.dotOne:SetPosition(0, dotOneY)
		listBoxItem.dotOne:SetMouseVisible(false)
	
		listBoxItem.hoverBox = Turbine.UI.Control()
		listBoxItem.hoverBox:SetParent(listBoxItem)
		listBoxItem.hoverBox:SetSize(20, 20)
		listBoxItem.hoverBox:SetPosition(-5, dotOneY - 5)
	
		listBoxItem.hoverBox.MouseEnter = function()
			graphWindow:ShowInfoGraph(graphWindow.dataCount)
			listBoxItem.dotOne:SetBackColor(graphWindow.dotSelectedColour)
		end
	
		listBoxItem.hoverBox.MouseLeave = function()
			graphWindow.infoWindow:SetVisible(false)
			listBoxItem.dotOne:SetBackColor(graphWindow.dotColour)
		end
		
	
		graphWindow.graph:AddItem(listBoxItem)
	end
	
	graphWindow.Unload = function()
		for i = 1, graphWindow.graph:GetItemCount() do
			local itemList = graphWindow.graph:GetItem(1):GetControls()
			for j = 1, itemList:GetCount() do
				itemList:RemoveAt(1)
			end
			graphWindow.graph:RemoveItemAt(1)
		end
		graphWindow.graph:ClearItems()
		graphWindow:SetParent(nil)
		local controlList = graphWindow:GetControls()
		for i = 1, controlList:GetCount() do
			controlList:RemoveAt(1)
		end
		for key, _ in pairs(graphWindow) do
			graphWindow[key] = nil
		end
		collectgarbage()
	end

    graphWindow:AddPositionItem()

    --[[
    graphWindow.data = {
        [1] = {1, 5},
        [2] = {2, 7},
        [3] = {3, 10},
        [4] = {4, 15},
        [5] = {5, 24.9},
        [6] = {6, 25},
        [7] = {7, 15},
        [8] = {8, 10},
        [9] = {9, 7},
        [10] = {10, 5},
    }

    graphWindow.data = {}

    for i = 1, 100 do
        graphWindow.data[i] = {i, math.random(0, 5000)}
    end
    --]]

    for _, value in pairs(graphWindow.data) do if value[2] > graphWindow.highestValue then graphWindow.highestValue = value[2] end end

    graphWindow.dataCount = Tablelength(graphWindow.data)
    for key, value in ipairs(graphWindow.data) do 
        if key == graphWindow.dataCount then break end
        local y1 = (1 - (graphWindow.data[key] and graphWindow.data[key][2] or 0) / (graphWindow.highestValue - graphWindow.lowestValue)) * (graphWindow.listBoxHeight - graphWindow.dotWidth)
        local y2 = (1 - (graphWindow.data[key + 1] and graphWindow.data[key + 1][2] or 0) / (graphWindow.highestValue - graphWindow.lowestValue)) * (graphWindow.listBoxHeight - graphWindow.dotWidth)

        self:ConstructGraphItem(graphWindow, y1, y2, value[1], key, value[2], key)
    end 

    graphWindow:GetEndItem()

    graphWindow.leftCover = Turbine.UI.Window()
    graphWindow.leftCover:SetParent(graphWindow)
    graphWindow.leftCover:SetVisible(false)
    graphWindow.leftCover:SetSize(graphWindow.itemWidth, graphWindow.listBoxHeight)
    graphWindow.leftCover:SetBackColor(Turbine.UI.Color.Black)
    graphWindow.leftCover:SetPosition(graphWindow.outerBorder, graphWindow.outerBorder + graphWindow.titleHeight)
    graphWindow.leftCover:SetMouseVisible(false)

    graphWindow.rightCover = Turbine.UI.Window()
    graphWindow.rightCover:SetParent(graphWindow)
    graphWindow.rightCover:SetVisible(false)
    graphWindow.rightCover:SetSize(graphWindow.itemWidth, graphWindow.listBoxHeight)
    graphWindow.rightCover:SetBackColor(Turbine.UI.Color.Black)
    graphWindow.rightCover:SetPosition(graphWindow.listBoxWidth + graphWindow.outerBorder - graphWindow.itemWidth, graphWindow.outerBorder + graphWindow.titleHeight)
    graphWindow.rightCover:SetMouseVisible(false)

    graphWindow.VisibleChanged = function()
        if graphWindow:IsVisible() then
            for i = 1, graphWindow.graph:GetItemCount() do
                local item = graphWindow.graph:GetItem(i)
                if item.Format then
                    item:Format()
                end
            end
        end
    end

	return graphWindow
end

function GraphDataWindow:ConstructGraphItem(graphWindow, dotOneY, dotTwoY, x, ID, showValue, index)
    local listBoxItem = Turbine.UI.Window()
    listBoxItem:SetVisible(true)
    listBoxItem:SetSize(graphWindow.itemWidth, graphWindow.listBoxHeight)
    --listBoxItem:SetBackColor(graphWindow.itemColour)
    listBoxItem:SetMouseVisible(false)
    listBoxItem.ID = ID
    listBoxItem.X = index
    listBoxItem.Value = showValue

    --local dotOneY = (1 - value / (graphWindow.highestValue - graphWindow.lowestValue)) * (graphWindow.listBoxHeight - graphWindow.dotWidth)
    --local dotTwoY = (1 - (nextValue or 1) / (graphWindow.highestValue - graphWindow.lowestValue)) * (graphWindow.listBoxHeight - graphWindow.dotWidth)

    if math.abs(dotOneY - dotTwoY) <= graphWindow.dotWidth then
        listBoxItem.line = Turbine.UI.Window()
        listBoxItem.line:SetParent(listBoxItem)
        listBoxItem.line:SetSize(graphWindow.itemWidth, graphWindow.lineWidth)
        listBoxItem.line:SetPosition(0, dotOneY)
        listBoxItem.line:SetBackColor(graphWindow.lineColour)
        --listBoxItem.line:SetBackColor(Turbine.UI.Color.White)
        listBoxItem.line:SetVisible(true)
        listBoxItem.line:SetMouseVisible(false)

        listBoxItem.PositionChanged = function()
            if graphWindow.minItem < listBoxItem.ID and graphWindow.maxItem > listBoxItem.ID then
                listBoxItem:SetVisible(true)
            else
                listBoxItem:SetVisible(false)
            end
        end

        listBoxItem.dotOne = Turbine.UI.Control()
        listBoxItem.dotOne:SetParent(listBoxItem)
        listBoxItem.dotOne:SetSize(graphWindow.dotWidth, graphWindow.dotWidth)
        listBoxItem.dotOne:SetBackColor(graphWindow.dotColour)
        listBoxItem.dotOne:SetPosition(0, dotOneY)
        listBoxItem.dotOne:SetMouseVisible(false)
    
        listBoxItem.hoverBox = Turbine.UI.Control()
        listBoxItem.hoverBox:SetParent(listBoxItem)
        listBoxItem.hoverBox:SetSize(20, 20)
        listBoxItem.hoverBox:SetPosition(-5, dotOneY - 5)
    
        listBoxItem.hoverBox.MouseEnter = function()
            graphWindow:ShowInfoGraph(listBoxItem.X)
            listBoxItem.dotOne:SetBackColor(graphWindow.dotSelectedColour)
        end
    
        listBoxItem.hoverBox.MouseLeave = function()
            graphWindow.infoWindow:SetVisible(false)
            listBoxItem.dotOne:SetBackColor(graphWindow.dotColour)
        end


        graphWindow.graph:AddItem(listBoxItem)
        return 
    end

    local yDiff =  math.pow(dotTwoY - dotOneY, 2)
    local lineLength = math.sqrt(graphWindow.itemWidthSquared + yDiff)



    listBoxItem.line = Turbine.UI.Window()
    listBoxItem.line:SetParent(listBoxItem)
    listBoxItem.line:SetSize(lineLength, lineLength)
    listBoxItem.line:SetVisible(true)
    listBoxItem.line:SetMouseVisible(false)

    listBoxItem.lineTexture = Turbine.UI.Control()
    listBoxItem.lineTexture:SetParent(listBoxItem.line)
    listBoxItem.lineTexture:SetSize(lineLength, graphWindow.lineWidth)
    listBoxItem.lineTexture:SetBackColor(graphWindow.lineColour)
    listBoxItem.lineTexture:SetMouseVisible(false)

    -- [[

    local firstProduct = math.pow(lineLength, 2) + graphWindow.itemWidthSquared - yDiff
    local secondProduct = 2 * lineLength * graphWindow.itemWidthAdjusted

    listBoxItem.lineAngle = math.deg(math.acos(firstProduct/secondProduct))

    local adjustment = (lineLength / math.sin(math.rad(90))) * math.sin(math.rad(180 - listBoxItem.lineAngle - 90))
    if dotOneY < dotTwoY then
        listBoxItem.line:SetRotation({x=0,y=0,z=-listBoxItem.lineAngle})
        listBoxItem.line:SetPosition(graphWindow.itemWidth / 2, dotOneY - adjustment / 2  )
        listBoxItem.lineTexture:SetPosition(0, lineLength - graphWindow.lineWidth)
    else
        listBoxItem.line:SetRotation({x=0,y=0,z=listBoxItem.lineAngle})
        listBoxItem.line:SetPosition(graphWindow.itemWidth / 2, dotTwoY + adjustment / 2  )
    end

    --]]

    listBoxItem.dotOne = Turbine.UI.Control()
    listBoxItem.dotOne:SetParent(listBoxItem)
    listBoxItem.dotOne:SetSize(graphWindow.dotWidth, graphWindow.dotWidth)
    listBoxItem.dotOne:SetBackColor(graphWindow.dotColour)
    listBoxItem.dotOne:SetPosition(0, dotOneY)
    listBoxItem.dotOne:SetMouseVisible(false)
    listBoxItem.dotOne:SetStretchMode(1)

    listBoxItem.hoverBox = Turbine.UI.Control()
    listBoxItem.hoverBox:SetParent(listBoxItem)
    listBoxItem.hoverBox:SetSize(20, 20)
    listBoxItem.hoverBox:SetPosition(-5, dotOneY - 5)

    listBoxItem.hoverBox.MouseEnter = function()
        graphWindow:ShowInfoGraph(listBoxItem.X)
        listBoxItem.dotOne:SetBackColor(graphWindow.dotSelectedColour)

    end

    listBoxItem.hoverBox.MouseLeave = function()
        graphWindow.infoWindow:SetVisible(false)
        listBoxItem.dotOne:SetBackColor(graphWindow.dotColour)
    end

    listBoxItem.Format = function()
        if graphWindow.minItem < listBoxItem.ID and graphWindow.maxItem > listBoxItem.ID then
            if not(listBoxItem:IsVisible()) then
                listBoxItem:SetVisible(true)
                listBoxItem.line:SetVisible(true)
                listBoxItem.dotOne:SetVisible(true)
                if dotOneY < dotTwoY then
                    listBoxItem.line:SetRotation({x=0,y=0,z=-listBoxItem.lineAngle})
                    listBoxItem.line:SetPosition(graphWindow.itemWidth / 2, dotOneY - adjustment / 2  )
                    listBoxItem.lineTexture:SetPosition(0, lineLength - graphWindow.lineWidth)
                else
                    listBoxItem.line:SetRotation({x=0,y=0,z=listBoxItem.lineAngle})
                    listBoxItem.line:SetPosition(graphWindow.itemWidth / 2, dotTwoY + adjustment / 2  )
                end
            end
        else
            listBoxItem:SetVisible(false)
            listBoxItem.line:SetVisible(false)
            listBoxItem.dotOne:SetVisible(false)
        end
    end

    listBoxItem.PositionChanged = function()
        listBoxItem:Format()
    end

    graphWindow.graph:AddItem(listBoxItem)
end

ListBoxCheckBox = class(Turbine.UI.Control)
function ListBoxCheckBox:Constructor(width, height, text, func)
    self.width = width
    self.height = height

	Turbine.UI.Control.Constructor(self)
    self:SetSize(self.width, self.height)
	self:SetBackColor(Turbine.UI.Color(0.39,0.75,0.75,0.75))
	self:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)

	self.checkBoxControl = Turbine.UI.Control()
	self.checkBoxControl:SetParent(self)
	self.checkBoxControl:SetSize(self.width, self.height - 2)
	self.checkBoxControl:SetBackColor(Turbine.UI.Color.Black)

	self.checkBox = Turbine.UI.Lotro.CheckBox()
	self.checkBox:SetParent(self.checkBoxControl)
	self.checkBox:SetSize(self.width, self.height - 2)
	self.checkBox:SetText(text)
	self.checkBox:SetPosition(5, 0)
	--self.checkBox:SetBackColor()
	
	self.checkBox.CheckedChanged = function()
		func(self.checkBox:IsChecked())
	end
end

function ListBoxCheckBox:SetForeColor(colour)
	self.checkBox:SetForeColor(colour)
end

function ListBoxCheckBox:SetChecked(value)
	self.checkBox:SetChecked(value)
end

GraphInfoWindow = class(Turbine.UI.Window)
function GraphInfoWindow:Constructor()
    Turbine.UI.Window.Constructor(self)
    self:SetSize(200, 100)
    self:SetVisible(false)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.TextBox = Turbine.UI.Label()
    self.TextBox:SetParent(self)
    self.TextBox:SetSize(200, 20)
    self.TextBox:SetPosition(3, 3)
    self.TextBox:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    self.TextBox:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
    --self.TextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.TextBox:SetMouseVisible(false)
    self.TextBox:SetBackColor(Turbine.UI.Color(0.8, 0.125, 0.125, 0.125))
end

function GraphInfoWindow:View(Text)
    self:SetVisible(true)
    local x, y = self:GetMousePosition()
    local oldX, oldY = self:GetPosition()
    self:SetPosition((x + oldX) - 75, y + oldY + 5)
    self:SetZOrder(1)
    self.TextBox:SetText(Text or "No info text available.")
    local length = Text and math.ceil(string.len(Text) / 70) + 1 or 1
    self:SetSize(200, length * 22)
    self.TextBox:SetSize(200 - 6, length * 22 - 6)
end

