AchievementHandlerI = class()
function AchievementHandlerI:Constructor()

	local loadedData = LoadTable(Turbine.DataScope.Account, "PrimeAchievements")

	if loadedData then
		self.achievementData = Compression:StringToTable(loadedData.A)
	else
		self.achievementData = {}
	end

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

	self.achievementScore = {
		Novice = 10,
		Intermediate = 20,
		Experienced = 30,
		Master = 40,
		Grandmaster = 50,
	}
end

function AchievementHandlerI:CompleteAchievement(name)
--[[
	Achievement structure
		name
			description
			category
			conditions
			progress
			completiondate
			difficulty
			rewards? points
			characters
]]

	-- If we find it we have already completed this achievement
	if self.achievementData[name] then
		-- We add our character to this achievement
		self.achievementData[name].Characters[self.player:GetName()] = true

		-- Add already done for account but not character frame
		--if Achievements then
		--	local achievementInfo = self:GetAchievementData()[name]
		--	Achievements:AddItem("", "Achievement Earned", achievementInfo.Difficulty, name, self.achievementScore[achievementInfo.Difficulty] or 10)
		--end
		return
	end

	local achievementInfo = self:GetAchievementData()[name]

	if not achievementInfo then return end

	-- First time we have done this achievement
	self.achievementData[name] = {
		Description = achievementInfo.Description,
		Category = achievementInfo.Category,
		Difficulty = achievementInfo.Difficulty,
		CompletionData = Turbine.Engine.GetDate(),
		Characters = {
			[self.player:GetName()] = true,
		}
	}

	-- Add first time frame
	if AchievementsPopup then
		--imagePath, headerText, titleText, achievementText, points
		AchievementsPopup:AddItem(achievementInfo.Icon or "", "Achievement Earned", achievementInfo.Difficulty, name, self.achievementScore[achievementInfo.Difficulty] or 10)
	end
end

function AchievementHandlerI:CheckAchievementProgress(category, progress)
	local categoryInfo = self:GetAchievementCategoryInfo()[category]
	if categoryInfo and progress then
		local achievementInfo = self:GetAchievementData()
		for achievementName, _ in pairs(categoryInfo) do
			if achievementInfo[achievementName] then
				if achievementInfo[achievementName].Goal <= progress then
					self:CompleteAchievement(achievementName)
				end
			end
		end
	end
end

function AchievementHandlerI:GetAchievementData()
	if self.achievementInfo then return self.achievementInfo end

	import "PrimePlugins.RaidTools.Achievements.AchievementInfo"
	self.achievementInfo = Compression:StringToTable(PrimePlugins.RaidTools.Achievements.GetAchievementInfo())

	return self.achievementInfo
end

function AchievementHandlerI:GetAchievementCategoryInfo()
	if self.achievementCategoryInfo then return self.achievementCategoryInfo end

	import "PrimePlugins.RaidTools.Achievements.AchievementInfo"
	self.achievementCategoryInfo = Compression:StringToTable(PrimePlugins.RaidTools.Achievements.GetAchievementCategoryInfo())

	return self.achievementCategoryInfo 
end

function AchievementHandlerI:GetAchievementsDone()
	return self.achievementData
end



function AchievementHandlerI:Unload()
	SaveTable(Turbine.DataScope.Account, "PrimeAchievements", {["A"] = Compression:TableToString(self.achievementData)})
end