PlayerSkillEvents = class(Turbine.Object)
function PlayerSkillEvents:Constructor()
    Turbine.Object.Constructor(self)

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.playerSkills = {}
    self.skillNames = {}
    self.skillCallbacks = {}

    self.mountedFilter = {
        ["Recuperate"] = 1091787641,
        ["Bee Swarm"] = 1091742016,
        ["Slash"] = 1091742735,
        ["Ferocious Roar"] = 1091787642,
        ["Execute"] = 1091744017,
    }

    self.SkillUpdate = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i=1,#event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end
    self:SetCallbacks()

    self.ExecuteUpdate = function(delta)
        if Turbine.Engine.GetGameTime() > self.reloadTime then
            self:Reload()
            if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
            self.updateCallback = nil
        end
    end


end

function PlayerSkillEvents:SetCallbacks()
    self.trainedSkills = self.player:GetTrainedSkills()
    for i = 1, self.trainedSkills:GetCount(), 1 do
        local item = self.trainedSkills:GetItem(i)
        local name = item:GetSkillInfo():GetName()

        if not(item:GetSkillInfo():GetIconImageID() == self.mountedFilter[name]) then 

            self.playerSkills[name] = item
            self.skillNames[name] = item:GetSkillInfo():GetIconImageID()

            self.skillCallbacks[name] =  AddCallback(self.playerSkills[name], "ResetTimeChanged", function(sender, args) 
                if self[name] and self.playerSkills[name] then
                    if type(self[name]) == "table" then
                        for key, value in pairs(self[name]) do
                            self.SkillUpdate(self, value, self.playerSkills[name])
                        end
                    elseif self[name] then
                        self.SkillUpdate(self, self[name], self.playerSkills[name])
                    end
                end
            end)
        end
    end
end

function PlayerSkillEvents:RemoveCallbacks()
    for name, callback in pairs(self.skillCallbacks) do
        RemoveCallback(self.playerSkills[name], "ResetTimeChanged", callback)
        self.skillCallbacks[name] = nil
    end

    self.skillCallbacks = nil
    self.playerSkills = nil
    self.trainedSkills = nil

    collectgarbage()

    self.playerSkills = {}
    self.skillCallbacks = {}
end

function PlayerSkillEvents:Update()
    if Turbine.Engine.GetGameTime() > self.reloadTime then
        self:Reload()
        self:SetWantsUpdates(false)
    end
end

function PlayerSkillEvents:GetSkill(name)
    return self.playerSkills[name]
end

function PlayerSkillEvents:ReloadWithDelay()
    self.reloadTime = Turbine.Engine.GetGameTime() + 3
    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
end

function PlayerSkillEvents:Reload()
    self:RemoveCallbacks()
    self:SetCallbacks()
end

function PlayerSkillEvents:SkillExists(name)
    if name == "Improved Power of Knowledge" then
        name = "Power of Knowledge"
    end
    return self.playerSkills[name] ~= nil
end

function PlayerSkillEvents:GetSkills()
    return self.skillNames
end 

function PlayerSkillEvents:CheckSkillCode(code)
    local skillInfo = self:GetSkillCodes()[self.player:GetClass()][code]

    if skillInfo then
        return skillInfo[1]
    end
end

function PlayerSkillEvents:GetSkillCodes()
    import "PrimePlugins.RaidTools.Handlers.SkillCodes"

    if not self.skillCodes then
        self.skillCodes = GetSkillCodes()
    end

    return self.skillCodes
end