NumericalInput = class(Turbine.UI.TreeNode)
function NumericalInput:Constructor(width, text, currentValue, changeFunction)
    Turbine.UI.TreeNode.Constructor(self)
    self.width = width
    self.height = 22

    self:SetSize(self.width, 22)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.innerBlock = Turbine.UI.Control()
    self.innerBlock:SetParent(self)
    self.innerBlock:SetSize(self.width, 20)
    self.innerBlock:SetPosition(1, 1)
    self.innerBlock:SetBackColor(Turbine.UI.Color(0.01,0.31,0.31))

    self.label = Turbine.UI.Label()
    self.label:SetParent(self.innerBlock)
    self.label:SetSize(150, 20)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(text)
    
    self.searchBar = Turbine.UI.Lotro.TextBox()
    self.searchBar:SetParent(self.innerBlock)
    self.searchBar:SetSize(100, 20)
    self.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.searchBar:SetPosition(150, 0)
    self.searchBar:SetText(currentValue)
    
    self.searchBar.TextChanged = function()
        if tonumber(self.searchBar:GetText()) then
            changeFunction(tonumber(self.searchBar:GetText())) 
        end        
    end

    self.SizeChanged = function ()
        self.width = self:GetWidth()
        self:Format()
    end
end

function NumericalInput:Format()
    self.innerBlock:SetSize(self.width, self.height)
end

function NumericalInput:SetValue(value)
    self.searchBar:SetText(tonumber(tostring(value)))
end

function NumericalInput:Unload()
    self.searchBar:SetParent(nil)
    self.label:SetParent(nil)
    self.innerBlock:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end

StringInput = class(Turbine.UI.TreeNode)
function StringInput:Constructor(width, text, currentValue, big, changeFunction)
    Turbine.UI.TreeNode.Constructor(self)
    self.width = width

    self.height = 22

    if big then self.height = 102 end

    self:SetSize(self.width, self.height)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.innerBlock = Turbine.UI.Control()
    self.innerBlock:SetParent(self)
    self.innerBlock:SetSize(self.width, self.height - 2)
    self.innerBlock:SetPosition(1, 1)
    self.innerBlock:SetBackColor(Turbine.UI.Color(0.01,0.31,0.31))

    self.label = Turbine.UI.Label()
    self.label:SetParent(self.innerBlock)
    self.label:SetSize(150, 20)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(text)
    
    self.searchBar = Turbine.UI.Lotro.TextBox()
    self.searchBar:SetParent(self.innerBlock)
    if big then
        self.searchBar:SetSize(300, 100)
    else
        self.searchBar:SetSize(self.width - 150 - 50, 20)
    end    
    self.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.searchBar:SetPosition(150, 0)
    self.searchBar:SetText(currentValue)
    
    self.searchBar.TextChanged = function()
        changeFunction(tostring(self.searchBar:GetText()))     
    end

    self.SizeChanged = function ()
        self.width = self:GetWidth()
        self:Format()
    end
end

function StringInput:Format()
    self.innerBlock:SetSize(self.width, self.height)
end

function StringInput:Unload()
    self.searchBar:SetParent(nil)
    self.label:SetParent(nil)
    self.innerBlock:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end

StringButtonInput = class(Turbine.UI.TreeNode)
function StringButtonInput:Constructor(width, text, currentValue, changeFunction)
    Turbine.UI.TreeNode.Constructor(self)
    self.width = width

    self.height = 400

    self:SetSize(self.width, self.height)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.innerBlock = Turbine.UI.Control()
    self.innerBlock:SetParent(self)
    self.innerBlock:SetSize(self.width, self.height - 2)
    self.innerBlock:SetPosition(1, 1)
    self.innerBlock:SetBackColor(Turbine.UI.Color(0.01,0.31,0.31))

    self.label = Turbine.UI.Label()
    self.label:SetParent(self.innerBlock)
    self.label:SetSize(150, 20)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(text)
    
    self.searchBar = Turbine.UI.Lotro.TextBox()
    self.searchBar:SetParent(self.innerBlock)
    self.searchBar:SetSize(400, 400) 
    self.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.searchBar:SetPosition(100, 0)
    self.searchBar:SetText(currentValue)

    self.inputButton = Turbine.UI.Lotro.Button()
    self.inputButton:SetParent(self.innerBlock)
	self.inputButton:SetSize(100, 20)
	self.inputButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.inputButton:SetPosition(0, 20)
    self.inputButton:SetText("Change")

    self.inputButton.Click = function()
        changeFunction(tostring(self.searchBar:GetText()))
    end

    self.SizeChanged = function ()
        self.width = self:GetWidth()
        self:Format()
    end
end

function StringButtonInput:Format()
    self.innerBlock:SetSize(self.width, self.height)
end

function StringButtonInput:Unload()
    self.searchBar:SetParent(nil)
    self.label:SetParent(nil)
    self.innerBlock:SetParent(nil)
    self.inputButton:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end

StringButtonInputSmall = class(Turbine.UI.TreeNode)
function StringButtonInputSmall:Constructor(width, text, currentValue, changeFunction)
    Turbine.UI.TreeNode.Constructor(self)
    self.width = width

    self.height = 20

    self:SetSize(self.width, self.height)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.innerBlock = Turbine.UI.Control()
    self.innerBlock:SetParent(self)
    self.innerBlock:SetSize(self.width, self.height - 2)
    self.innerBlock:SetPosition(1, 1)
    self.innerBlock:SetBackColor(Turbine.UI.Color(0.01,0.31,0.31))

    self.label = Turbine.UI.Label()
    self.label:SetParent(self.innerBlock)
    self.label:SetSize(150, 20)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(text)
    
    self.searchBar = Turbine.UI.Lotro.TextBox()
    self.searchBar:SetParent(self.innerBlock)
    self.searchBar:SetSize(400, self.height) 
    self.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.searchBar:SetPosition(100, 0)
    self.searchBar:SetText(currentValue)

    self.inputButton = Turbine.UI.Lotro.Button()
    self.inputButton:SetParent(self.innerBlock)
	self.inputButton:SetSize(100, 20)
	self.inputButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.inputButton:SetPosition(500, 0)
    self.inputButton:SetText("Change")

    self.inputButton.Click = function()
        changeFunction(tostring(self.searchBar:GetText()))
    end

    self.SizeChanged = function ()
        self.width = self:GetWidth()
        self:Format()
    end
end

function StringButtonInputSmall:Format()
    self.innerBlock:SetSize(self.width, self.height)
end

function StringButtonInputSmall:Unload()
    self.searchBar:SetParent(nil)
    self.label:SetParent(nil)
    self.innerBlock:SetParent(nil)
    self.inputButton:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end

IconInput = class(Turbine.UI.TreeNode)
function IconInput:Constructor(width, text, currentValue, options, changeFunction)
    Turbine.UI.TreeNode.Constructor(self)
    self.width = width
    self.height = 42

    self:SetSize(self.width, 42)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.innerBlock = Turbine.UI.Control()
    self.innerBlock:SetParent(self)
    self.innerBlock:SetSize(self.width, 40)
    self.innerBlock:SetPosition(1, 1)
    self.innerBlock:SetBackColor(Turbine.UI.Color(0.01,0.31,0.31))

    self.label = Turbine.UI.Label()
    self.label:SetParent(self.innerBlock)
    self.label:SetSize(150, 20)
    self.label:SetPosition(0, 10)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(text)
    
    self.EffectTimerIconBox = Turbine.UI.Control()
    self.EffectTimerIconBox:SetParent(self.innerBlock)
    self.EffectTimerIconBox:SetPosition(50, 4)
    self.EffectTimerIconBox:SetSize(32,32)
    self.EffectTimerIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.EffectTimerIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(currentValue) or currentValue))
    self.EffectTimerIconBox:SetVisible(true)

    self.TimerIconTextBox = Turbine.UI.Lotro.TextBox()
    self.TimerIconTextBox:SetParent(self.innerBlock)
    self.TimerIconTextBox:SetSize(250, 20)
    self.TimerIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.TimerIconTextBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.TimerIconTextBox:SetMouseVisible(false)
    self.TimerIconTextBox:SetPosition(90, 10)
    self.TimerIconTextBox:SetMultiline(false)
    self.TimerIconTextBox:SetText(currentValue)

    self.EffectTimerIconBox.MouseClick = function()
        local iconWindow = options:GetIconWindow()
        iconWindow:SetVisible(true) 
        iconWindow:Register(self.EffectTimerIconBox, self.TimerIconTextBox)
    end

    self.EffectTimerIconBox.MouseDoubleClick = function()
        self.EffectTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.TimerIconTextBox:SetText(0x41007F41)
    end

    self.Button = Turbine.UI.Lotro.Button()
    self.Button:SetParent(self)
    self.Button:SetSize(150, 10)
    self.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.Button:SetMouseVisible(false)
    self.Button:SetPosition(345, 10)
    self.Button:SetText("Set New Icon")

    self.Button.Click = function()
        changeFunction(tostring(self.TimerIconTextBox:GetText()))
    end

    self.SizeChanged = function ()
        self.width = self:GetWidth()
        self:Format()
    end
end

function IconInput:Format()
    self.innerBlock:SetSize(self.width, self.height)
end

function IconInput:Unload()
    self.label:SetParent(nil)
    self.innerBlock:SetParent(nil)
    self.TimerIconTextBox:SetParent(nil)
    self.EffectTimerIconBox:SetParent(nil)
    self.Button:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end

BoolInput = class(Turbine.UI.TreeNode)
function BoolInput:Constructor(width, text, currentValue, changeFunction, backgroundColour)
    Turbine.UI.TreeNode.Constructor(self)
    self.width = width
    self.height = 22

    self:SetSize(self.width, 22)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.innerBlock = Turbine.UI.Control()
    self.innerBlock:SetParent(self)
    self.innerBlock:SetSize(self.width, 20)
    self.innerBlock:SetPosition(1, 1)
    self.innerBlock:SetBackColor(backgroundColour or Turbine.UI.Color(0.01,0.31,0.31))

    self.label = Turbine.UI.Lotro.CheckBox()
    self.label:SetParent(self.innerBlock)
    self.label:SetSize(width, 20)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(text)
    self.label:SetChecked(currentValue)
    
    self.label.CheckedChanged = function()
        changeFunction(self.label:IsChecked())    
    end

    self.SizeChanged = function ()
        self.width = self:GetWidth()
        self:Format()
    end
end

function BoolInput:Format()
    self.innerBlock:SetSize(self.width, self.height)
    self.label:SetSize(self.width, self.height)
end

function BoolInput:SetChecked(value)
    if self.label then self.label:SetChecked(value) end
end

function BoolInput:IsChecked()
    return self.label:IsChecked()
end

function BoolInput:Unload()
    self.label:SetParent(nil)
    self.innerBlock:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end

FontInput = class(Turbine.UI.TreeNode)
function FontInput:Constructor(width, text, currentValue, changeFunction)
    Turbine.UI.TreeNode.Constructor(self)
    self.width = width

    self:SetSize(self.width, 22)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.innerBlock = Turbine.UI.Control()
    self.innerBlock:SetParent(self)
    self.innerBlock:SetSize(self.width, 20)
    self.innerBlock:SetPosition(1, 1)
    self.innerBlock:SetBackColor(Turbine.UI.Color(0.01,0.31,0.31))

    self.label = Turbine.UI.Label()
    self.label:SetParent(self.innerBlock)
    self.label:SetSize(150, 20)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(text)

    local menu = Turbine.UI.ContextMenu()
    local menuItems = menu:GetItems()

    local sortedKeys = {}
    for key, _ in pairs(Turbine.UI.Lotro.Font) do table.insert( sortedKeys, key ) end
    table.sort( sortedKeys )

    self.Button = Turbine.UI.Lotro.Button()
    self.Button:SetParent(self)
    self.Button:SetSize(150, 100)
    self.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.Button:SetMouseVisible(false)
    self.Button:SetPosition(150, 0)
    self.Button:SetText(currentValue)

    for key, name in pairs(sortedKeys) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            self.Button:SetText(name)
            changeFunction(name)
        end
        
        menuItems:Add(menuItem)
    end

    self.Button.Click = function()
        menu:ShowMenu()
    end

    self.SizeChanged = function ()
        self.width = self:GetWidth()
        self:Format()
    end
end

function FontInput:Format()
    self.innerBlock:SetSize(self.width, 20)
end

function FontInput:Unload()
    self.label:SetParent(nil)
    self.innerBlock:SetParent(nil)
    self.Button:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end

AlignmentInput = class(Turbine.UI.TreeNode)
function AlignmentInput:Constructor(width, text, currentValue, options, changeFunction)
    Turbine.UI.TreeNode.Constructor(self)
    self.width = width

    self:SetSize(self.width, 22)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.innerBlock = Turbine.UI.Control()
    self.innerBlock:SetParent(self)
    self.innerBlock:SetSize(self.width, 20)
    self.innerBlock:SetPosition(1, 1)
    self.innerBlock:SetBackColor(Turbine.UI.Color(0.01,0.31,0.31))

    self.label = Turbine.UI.Label()
    self.label:SetParent(self.innerBlock)
    self.label:SetSize(250, 20)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(text)

    local menu = Turbine.UI.ContextMenu()
    local menuItems = menu:GetItems()

    local sortedKeys = {}
    for key, _ in pairs(options) do table.insert( sortedKeys, key ) end
    table.sort( sortedKeys )

    self.Button = Turbine.UI.Lotro.Button()
    self.Button:SetParent(self)
    self.Button:SetSize(150, 100)
    self.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.Button:SetMouseVisible(false)
    self.Button:SetPosition(250, 0)
    self.Button:SetText(currentValue)

    for key, name in pairs(sortedKeys) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            self.Button:SetText(name)
            changeFunction(name)
        end
        
        menuItems:Add(menuItem)
    end

    self.Button.Click = function()
        menu:ShowMenu()
    end

    self.SizeChanged = function ()
        self.width = self:GetWidth()
        self:Format()
    end
end

function AlignmentInput:Format()
    self.innerBlock:SetSize(self.width, 20)
end

function AlignmentInput:Unload()
    self.label:SetParent(nil)
    self.innerBlock:SetParent(nil)
    self.Button:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end

ButtonInput = class(Turbine.UI.TreeNode)
function ButtonInput:Constructor(width, text, func)
    Turbine.UI.TreeNode.Constructor(self)
    self.width = width

    self:SetSize(self.width, 22)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.innerBlock = Turbine.UI.Control()
    self.innerBlock:SetParent(self)
    self.innerBlock:SetSize(self.width, 20)
    self.innerBlock:SetPosition(1, 1)
    self.innerBlock:SetBackColor(Turbine.UI.Color(0.01,0.31,0.31))

    self.label = Turbine.UI.Label()
    self.label:SetParent(self.innerBlock)
    self.label:SetSize(150, 20)
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(text)
    
    self.button = Turbine.UI.Lotro.Button()
    self.button:SetParent(self.innerBlock)
    self.button:SetSize(300, 20)
    self.button:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.button:SetPosition(150, 0)
    
    self.button.Click = function()
        func()     
    end

    self.SizeChanged = function ()
        self.width = self:GetWidth()
        self:Format()
    end
end

function ButtonInput:SetText(text)
    self.button:SetText(text)
end

function ButtonInput:GetText()
    return self.button:GetText()
end


function ButtonInput:Format()
    self.innerBlock:SetSize(self.width, 20)
end

function ButtonInput:Unload()
    self.label:SetParent(nil)
    self.innerBlock:SetParent(nil)
    self.button:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end


HeaderNode = class(Turbine.UI.TreeNode)
function HeaderNode:Constructor(width, text, alignment, font, backgroundColour)
    Turbine.UI.TreeNode.Constructor(self)
    self.width = width

    self:SetSize(self.width, 27)
    self:SetPosition(0, 0)
    self:SetMouseVisible(true)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.innerBlock = Turbine.UI.Control()
    self.innerBlock:SetParent(self)
    self.innerBlock:SetSize(self.width, 25)
    self.innerBlock:SetPosition(1, 1)
    self.innerBlock:SetBackColor(backgroundColour or Turbine.UI.Color(0.01,0.31,0.31))
    self.innerBlock:SetMouseVisible(false)

    self.label = Turbine.UI.Label()
    self.label:SetParent(self.innerBlock)
    self.label:SetSize(width, 25)
    self.label:SetFont(font or Turbine.UI.Lotro.Font.BookAntiquaBold22)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(alignment or Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(text)
    self.label:SetMouseVisible(false)

    self.SizeChanged = function ()
        self.width = self:GetWidth()
        self:Format()
    end
end

function HeaderNode:Format()
    self.innerBlock:SetSize(self.width, 25)
    self.label:SetSize(self.width, 25)
end

function HeaderNode:Unload()
    if self.label then self.label:SetParent(nil) end
    if self.innerBlock then self.innerBlock:SetParent(nil) end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end

function SingleColourOptions(parent, width, text, currentValue, func, index)
    local y = 0
    if index then
        y = (index - 1) * 130
    end
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetParent(parent)
    valuebar:SetSize(width, 130)
    valuebar:SetPosition(0, y)
    valuebar:SetMouseVisible(false)
    valuebar:SetBackColor(Turbine.UI.Color(0.01,0.31,0.31))

    valuebar.fontColour = Turbine.UI.Color.Black

    valuebar.labelBorder = Turbine.UI.Control()
    valuebar.labelBorder:SetParent(valuebar)
    valuebar.labelBorder:SetSize(250, 24)
    valuebar.labelBorder:SetMouseVisible(false)
    valuebar.labelBorder:SetPosition(5, 5)
    valuebar.labelBorder:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.labelBorder:SetBackColor(Turbine.UI.Color.Black)

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar.labelBorder)
	valuebar.label:SetSize(250 - 4, 20)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(2, 2)
    valuebar.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    valuebar.label:SetForeColor(valuebar.fontColour)
    valuebar.label:SetBackColor(Turbine.UI.Color.DarkGray)
    valuebar.label:SetText(string.format("  %s", text))

    valuebar.currentBorder = Turbine.UI.Control()
    valuebar.currentBorder:SetParent(valuebar)
    valuebar.currentBorder:SetSize(110 + 4, 36 + 4)
    valuebar.currentBorder:SetMouseVisible(false)
    valuebar.currentBorder:SetPosition(365, 30)
    valuebar.currentBorder:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.currentBorder:SetBackColor(Turbine.UI.Color.Black)

    valuebar.currentBackground = Turbine.UI.Control()
    valuebar.currentBackground:SetParent(valuebar.currentBorder)
    valuebar.currentBackground:SetSize(110, 36)
    valuebar.currentBackground:SetMouseVisible(false)
    valuebar.currentBackground:SetPosition(2, 2)
    valuebar.currentBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.currentBackground:SetBackColor(Turbine.UI.Color.DarkGray)

    valuebar.currentLabel = Turbine.UI.Label()
	valuebar.currentLabel:SetParent(valuebar.currentBackground)
	valuebar.currentLabel:SetSize(60, 20)
	valuebar.currentLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.currentLabel:SetPosition(0, 10)
    valuebar.currentLabel:SetForeColor(valuebar.fontColour)
    valuebar.currentLabel:SetText("Current:")

    valuebar.CurrentBackground = Turbine.UI.Control()
    valuebar.CurrentBackground:SetParent(valuebar.currentBackground)
    valuebar.CurrentBackground:SetSize(32, 32)
    valuebar.CurrentBackground:SetMouseVisible(false)
    valuebar.CurrentBackground:SetPosition(70, 2)
    valuebar.CurrentBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.CurrentBackground:SetBackColor(Turbine.UI.Color.Black)

    valuebar.Current = Turbine.UI.Control()
    valuebar.Current:SetParent(valuebar.CurrentBackground)
    valuebar.Current:SetSize(28, 28)
    valuebar.Current:SetMouseVisible(false)
    valuebar.Current:SetPosition(2, 2)
    valuebar.Current:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.Current:SetBackColor(Turbine.UI.Color(currentValue[1], currentValue[2], currentValue[3], currentValue[4]))

    valuebar.newBorder = Turbine.UI.Control()
    valuebar.newBorder:SetParent(valuebar)
    valuebar.newBorder:SetSize(110 + 4, 36 + 4)
    valuebar.newBorder:SetMouseVisible(false)
    valuebar.newBorder:SetPosition(365, 84)
    valuebar.newBorder:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.newBorder:SetBackColor(Turbine.UI.Color.Black)

    valuebar.newBackground = Turbine.UI.Control()
    valuebar.newBackground:SetParent(valuebar.newBorder)
    valuebar.newBackground:SetSize(110, 36)
    valuebar.newBackground:SetMouseVisible(false)
    valuebar.newBackground:SetPosition(2, 2)
    valuebar.newBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.newBackground:SetBackColor(Turbine.UI.Color.DarkGray)

    valuebar.newLabel = Turbine.UI.Label()
	valuebar.newLabel:SetParent(valuebar.newBackground)
	valuebar.newLabel:SetSize(60, 20)
	valuebar.newLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.newLabel:SetForeColor(valuebar.fontColour)
	valuebar.newLabel:SetPosition(0, 10)
    valuebar.newLabel:SetText("New:")

    valuebar.ExampleBackground = Turbine.UI.Control()
    valuebar.ExampleBackground:SetParent(valuebar.newBackground)
    valuebar.ExampleBackground:SetSize(32, 32)
    valuebar.ExampleBackground:SetMouseVisible(false)
    valuebar.ExampleBackground:SetPosition(70, 2)
    valuebar.ExampleBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.ExampleBackground:SetBackColor(Turbine.UI.Color.Black)

    valuebar.Example = Turbine.UI.Control()
    valuebar.Example:SetParent(valuebar.ExampleBackground)
    valuebar.Example:SetSize(28, 28)
    valuebar.Example:SetMouseVisible(false)
    valuebar.Example:SetPosition(2, 2)
    valuebar.Example:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.Example:SetBackColor(Turbine.UI.Color(currentValue[1], currentValue[2], currentValue[3], currentValue[4]))

    valuebar.ScrollbarBackgroundBorder = Turbine.UI.Control()
    valuebar.ScrollbarBackgroundBorder:SetParent(valuebar)
    valuebar.ScrollbarBackgroundBorder:SetSize(350 + 4, 90 + 4)
    valuebar.ScrollbarBackgroundBorder:SetMouseVisible(false)
    valuebar.ScrollbarBackgroundBorder:SetPosition(5, 30)
    valuebar.ScrollbarBackgroundBorder:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.ScrollbarBackgroundBorder:SetBackColor(Turbine.UI.Color.Black)

    valuebar.ScrollbarBackground = Turbine.UI.Control()
    valuebar.ScrollbarBackground:SetParent(valuebar.ScrollbarBackgroundBorder)
    valuebar.ScrollbarBackground:SetSize(350, 90)
    valuebar.ScrollbarBackground:SetMouseVisible(false)
    valuebar.ScrollbarBackground:SetPosition(2, 2)
    valuebar.ScrollbarBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.ScrollbarBackground:SetBackColor(Turbine.UI.Color.DarkGray)

    -- 1
    valuebar.scrollbar1 = Turbine.UI.Lotro.ScrollBar()
	valuebar.scrollbar1:SetParent(valuebar.ScrollbarBackground)
    valuebar.scrollbar1:SetValue(currentValue[1] * 100)
	valuebar.scrollbar1:SetSize(250, 10)
    valuebar.scrollbar1:SetOrientation(Turbine.UI.Orientation.Horizontal)
	valuebar.scrollbar1:SetPosition(10, 10)
    valuebar.scrollbar1:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.scrollbar1:SetBackColor(Turbine.UI.Color(currentValue[1], 0, 0, 0))

    valuebar.scrollbar1label = Turbine.UI.Label()
	valuebar.scrollbar1label:SetParent(valuebar.ScrollbarBackground)
	valuebar.scrollbar1label:SetSize(150, 20)
	valuebar.scrollbar1label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.scrollbar1label:SetPosition(260, 5)
    valuebar.scrollbar1label:SetForeColor(valuebar.fontColour)
    valuebar.scrollbar1label:SetText("Alpha: " .. tostring(currentValue[1]))

    -- 2
    valuebar.scrollbar2 = Turbine.UI.Lotro.ScrollBar()
	valuebar.scrollbar2:SetParent(valuebar.ScrollbarBackground)
	valuebar.scrollbar2:SetSize(250, 10)
    valuebar.scrollbar2:SetOrientation(Turbine.UI.Orientation.Horizontal)
	valuebar.scrollbar2:SetPosition(10, 30)
    valuebar.scrollbar2:SetValue(currentValue[2] * 100)
    valuebar.scrollbar2:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.scrollbar2:SetBackColor(Turbine.UI.Color(1, currentValue[2], 0, 0))

    valuebar.scrollbar2label = Turbine.UI.Label()
	valuebar.scrollbar2label:SetParent(valuebar.ScrollbarBackground)
	valuebar.scrollbar2label:SetSize(150, 20)
	valuebar.scrollbar2label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.scrollbar2label:SetPosition(260, 25)
    valuebar.scrollbar2label:SetForeColor(valuebar.fontColour)
    valuebar.scrollbar2label:SetText("Red: " .. tostring(currentValue[2]))
    
    -- 3
    valuebar.scrollbar3 = Turbine.UI.Lotro.ScrollBar()
	valuebar.scrollbar3:SetParent(valuebar.ScrollbarBackground)
	valuebar.scrollbar3:SetSize(250, 10)
    valuebar.scrollbar3:SetOrientation(Turbine.UI.Orientation.Horizontal)
	valuebar.scrollbar3:SetPosition(10, 50)
    valuebar.scrollbar3:SetValue(currentValue[3] * 100)
    valuebar.scrollbar3:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.scrollbar3:SetBackColor(Turbine.UI.Color(1, 0, currentValue[3], 0))

    valuebar.scrollbar3label = Turbine.UI.Label()
	valuebar.scrollbar3label:SetParent(valuebar.ScrollbarBackground)
	valuebar.scrollbar3label:SetSize(150, 20)
	valuebar.scrollbar3label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.scrollbar3label:SetPosition(260, 45)
    valuebar.scrollbar3label:SetForeColor(valuebar.fontColour)
    valuebar.scrollbar3label:SetText("Green: " .. tostring(currentValue[3]))

    -- 4
    valuebar.scrollbar4 = Turbine.UI.Lotro.ScrollBar()
    valuebar.scrollbar4:SetParent(valuebar.ScrollbarBackground)
    valuebar.scrollbar4:SetSize(250, 10)
    valuebar.scrollbar4:SetOrientation(Turbine.UI.Orientation.Horizontal)
    valuebar.scrollbar4:SetPosition(10, 70)
    valuebar.scrollbar4:SetValue(currentValue[4] * 100)
    valuebar.scrollbar4:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.scrollbar4:SetBackColor(Turbine.UI.Color(1, 0, 0, currentValue[4]))

    valuebar.scrollbar4label = Turbine.UI.Label()
    valuebar.scrollbar4label:SetParent(valuebar.ScrollbarBackground)
    valuebar.scrollbar4label:SetSize(150, 20)
    valuebar.scrollbar4label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.scrollbar4label:SetPosition(260, 65)
    valuebar.scrollbar4label:SetForeColor(valuebar.fontColour)
    valuebar.scrollbar4label:SetText("Blue: " .. tostring(currentValue[4]))

    valuebar.scrollbar1.ValueChanged = function()
        valuebar.scrollbar1label:SetText("Alpha:  " .. valuebar.scrollbar1:GetValue() / 100)
        valuebar.Example:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100))
        valuebar.scrollbar1:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, 0, 0, 0))
    end

    valuebar.scrollbar2.ValueChanged = function()
        valuebar.scrollbar2label:SetText("Red:  " .. valuebar.scrollbar2:GetValue() / 100)
        valuebar.Example:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100))
        valuebar.scrollbar2:SetBackColor(Turbine.UI.Color(1, valuebar.scrollbar2:GetValue() / 100, 0, 0))
    end

    valuebar.scrollbar3.ValueChanged = function()
        valuebar.scrollbar3label:SetText("Green:  " .. valuebar.scrollbar3:GetValue() / 100)
        valuebar.Example:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100))
        valuebar.scrollbar3:SetBackColor(Turbine.UI.Color(1, 0, valuebar.scrollbar3:GetValue() / 100, 0))
    end


    valuebar.scrollbar4.ValueChanged = function()
        valuebar.scrollbar4label:SetText("Blue:  " .. valuebar.scrollbar4:GetValue() / 100)
        valuebar.Example:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100))
        valuebar.scrollbar4:SetBackColor(Turbine.UI.Color(1, 0, 0, valuebar.scrollbar4:GetValue() / 100))
    end

    valuebar.buttonBorder = Turbine.UI.Control()
    valuebar.buttonBorder:SetParent(valuebar)
    valuebar.buttonBorder:SetSize(160 + 4, 20 + 4)
    valuebar.buttonBorder:SetMouseVisible(false)
    valuebar.buttonBorder:SetPosition(315, 5)
    valuebar.buttonBorder:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.buttonBorder:SetBackColor(Turbine.UI.Color.Black)

    valuebar.buttonBackground = Turbine.UI.Control()
    valuebar.buttonBackground:SetParent(valuebar.buttonBorder)
    valuebar.buttonBackground:SetSize(160, 20)
    valuebar.buttonBackground:SetMouseVisible(false)
    valuebar.buttonBackground:SetPosition(2, 2)
    valuebar.buttonBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    valuebar.buttonBackground:SetBackColor(Turbine.UI.Color.DarkGray)

    valuebar.Button = Turbine.UI.Lotro.Button()
    valuebar.Button:SetParent(valuebar.buttonBackground)
    valuebar.Button:SetSize(150, 100)
    valuebar.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.Button:SetMouseVisible(false)
    valuebar.Button:SetPosition(5, 0)
    valuebar.Button:SetText("Change colour")

    valuebar.Button.Click = function()
        valuebar.Current:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100))
        func({valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100}, index)
    end

    valuebar.Unload = function()
        RecursiveDeletion(valuebar)
    end

    valuebar.SizeChanged = function()
        
    end
end