OptionsHeader = class(Turbine.UI.TreeNode)
function OptionsHeader:Constructor(width, itemCount, doNotRememberSelected)
    Turbine.UI.TreeNode.Constructor(self)

    self.width = width
    self.itemHeight = 30
    self.borderWidth = 1
    self.itemCount = itemCount
    self.rememberSelected = not(doNotRememberSelected)

    self:SetSize(self.width, 50)

    self.border = Turbine.UI.Control()
    self.border:SetParent(self)
    self.border:SetSize(self.width, 50)
    self.border:SetMouseVisible(false)
    self.border:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.listbox = Turbine.UI.ListBox()
    self.listbox:SetParent(self)
    self.listbox:SetSize(self.width, 50)
    self.listbox:SetPosition(self.borderWidth, self.borderWidth)
    self.listbox:SetMouseVisible(false)
    self.listbox:SetBackColor(Turbine.UI.Color(0, 0, 0))
    self.listbox:SetMaxItemsPerLine(self.itemCount)
    self.listbox:SetOrientation(Turbine.UI.Orientation.Horizontal)

    self.selectedItem = nil

    self.listbox.SelectedIndexChanged = function(sender, args)
        if self.rememberSelected then
            if self.selectedItem then
                self.selectedItem:SetSelected(false)
            end
            self.listbox:GetSelectedItem():SetSelected(true)
            self.selectedItem = self.listbox:GetSelectedItem()
        end
    end

    self:Resize()

    self.SizeChanged = function()
        self.width = self:GetWidth()
        self.listbox:SetWidth(self.width)
        self:Resize()
    end
end

function OptionsHeader:AddItem(item)
    self.listbox:AddItem(item)
    self:Resize()
end

function OptionsHeader:Unselect()
    if self.rememberSelected then
        if self.selectedItem then
            self.selectedItem:SetSelected(false)
        end
    end
end

function OptionsHeader:Resize()
    local height = math.ceil(self.listbox:GetItemCount() / self.itemCount) * self.itemHeight
    self:SetSize(self.width, height)
    self.border:SetSize(self.width, height)
    self.listbox:SetSize(self.width - 2 * self.borderWidth, height - 2 * self.borderWidth)
end

function OptionsHeader:ResizeItems()
    for i = 1, self.listbox:GetItemCount() do
        self.listbox:GetItem(i):SetWidth(self.width/self.itemCount)
    end
end

function OptionsHeader:Unload()
    for j = 1, self.listbox:GetItemCount() do
        self.listbox:GetItem(j):Unload()
    end
    self.listbox:ClearItems()
    self.border:SetParent(nil)
    self.listbox:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
end

OptionsListBoxButton = class(Turbine.UI.Control)
function OptionsListBoxButton:Constructor(width, height, name, func)
    Turbine.UI.Control.Constructor(self)
    self.width = width
    self.height = height
    self.borderWidth = 1
    self.selected = false
    self.name = name

    self:SetSize(self.width, self.height)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.label = Turbine.UI.Label()
    self.label:SetParent(self)
    self.label:SetSize(self.width - 2 * self.borderWidth, self.height - 2 * self.borderWidth)
    self.label:SetPosition(self.borderWidth, self.borderWidth)
    self.label:SetBackColor(Turbine.UI.Color(0.07,0.53,0.53))
    self.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.label:SetOutlineColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.label:SetText(name)

    self.label.MouseEnter = function()
        if self.label then
            self.label:SetBackColor(Turbine.UI.Color.DarkSlateGray)
        end
    end

    self.label.MouseLeave = function()
        if not(self.selected) and self.label then
            self.label:SetBackColor(Turbine.UI.Color(0.07,0.53,0.53))
        end
    end

    self.label.MouseClick = function()
        func()
    end

    self.SizeChanged = function()
        self.width, self.height = self:GetSize()
        self:Format()
    end
end

function OptionsListBoxButton:SetSelected(value)
    if value then
        self.selected = true
        self.label:SetBackColor(Turbine.UI.Color.DarkSlateGray)
    else
        self.selected = false
        self.label:SetBackColor(Turbine.UI.Color(0.07,0.53,0.53))
    end
end

function OptionsListBoxButton:Format()
    self.label:SetSize(self.width - 2 * self.borderWidth, self.height - 2 * self.borderWidth)
    self.label:SetPosition(self.borderWidth, self.borderWidth)
end

function OptionsListBoxButton:Unload()
    self.label:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
    --self:SetParent(nil)

end