SkillBar = class(Turbine.UI.Control)
function SkillBar:Constructor(parent, x, y, alignment, useStatic, scale, eventList, offset)
    Turbine.UI.Control.Constructor(self)
    self:SetParent(parent)
    self:SetPosition(x, y)
    self:SetMouseVisible(false)
	--[[
		Bar features:
			Static and priority positioning
			Empty quickslots?
			Multi of the same quickslot
			Give quickslot IDs
			Accept events for cooldown, hiding, visible, opacity, highlight.
			Spin?
			Cooldown text alignment and font
			Label size/scale
			Bar alignment

		Quickslot features:
			Highlight (+event)
			Opacity (+event)
			Visible (+event)
			Keybind
			Cooldown (+event)
			Circle cooldown?
			Square cooldown?
			If item detect if empty?
			Cooldown text alignment and font override
			Only show if available?

			Restrict to only positive positions (0 or 1 index?)
	]]
	self.alignment = alignment
	self.useStatic = useStatic
	self.scale = scale
	self.eventList = eventList

	self.offset = offset or 1
	self.labelSize = (34 * self.scale) + self.offset

	-- Saves all quickslots in the bar
	self.quickslots = {}

	-- If true the positioning will be strict otherwise it will be based on priority
	self.staticMode = false

	self.xCounts = {}
	self.yCounts = {}

	self.xMax = 0
	self.yMax = 0
	self.xMaxCount = 0

	self.quickslotEvents = {}
end

function SkillBar:AddQuickslot(ID, quickslot, x, y)
    if self.quickslots[ID] then
		ThrowException(string.format("Duplicate quickslot ID: %s, X: %s, Y: %s", tostring(ID), tostring(x), tostring(y)))
        return
    end

	if not ID or not quickslot or not x or not y then
		ThrowException(string.format("Null value ID: %s, X: %s, Y: %s, Quickslot: %s", tostring(ID), tostring(x), tostring(y), tostring(quickslot)))
        return
	end

	self.quickslots[ID] = {
		["Quickslot"] = quickslot,
		["X"] = x,
		["Y"] = y,
	}

    -- Update counts
    if self.xCounts[y] then
        self.xCounts[y] = self.xCounts[y] + 1
    else
        self.xCounts[y] = 1
    end

	if self.yCounts[x] then
        self.yCounts[x] = self.yCounts[x] + 1
    else
        self.yCounts[x] = 1
    end

	self.yMax = math.max(self.yMax, y)
	self.xMax = math.max(self.xMax, x)
	self.xMaxCount = math.max(self.xMaxCount, self.xCounts[y])
	
end

function SkillBar:RemoveQuickslot(ID)
	if not self.quickslots[ID] then
		ThrowException(string.format("Trying to remove non existing quickslot ID: %s", tostring(ID)))
		return
	end

	-- Update counts
	self.xCounts[self.quickslots[ID].Y] = self.xCounts[self.quickslots[ID].Y] - 1
	self.yCounts[self.quickslots[ID].X] = self.yCounts[self.quickslots[ID].X] - 1

	-- Delete from bar and clean
	self.quickslots[ID] = nil
end

function SkillBar:PerformLayout()
	local maxXCount = 0
	self.width = self.xMaxCount * self.labelSize
	for y, count in spairs(self.xCounts, function(t, a, b) return a < b end) do
		local xPositionPriority = 0
		for ID, info in spairs(self.quickslots, function(t, a, b) return t[a].X < t[b].X end) do	
			if info.Y == y then
				local xPos, yPos = self:GetLabelPosition(info.X, info.Y, xPositionPriority)
				self.quickslots[ID].Quickslot:SetPosition(xPos, yPos)
				if not self.useStatic then
					xPositionPriority = xPositionPriority + 1
				end
			end
		end
		maxXCount = math.max(maxXCount, xPositionPriority)
	end

	if self.alignment == "Left" then
		maxXCount = maxXCount - 1
		self.xMax = self.xMax - 1
	end

	if not self.useStatic then
		self.width = maxXCount * self.labelSize
	end

	self:SetSize(self.useStatic and (self.labelSize * self.scale * (self.xMax + 1)) or self.labelSize * self.scale * maxXCount,  self.labelSize * self.scale * self.yMax)
	self:SetEventInfo()
end

function SkillBar:SetEventInfo()
	self.quickslotEvents = {}

	-- Prepare events
	for ID, quickslotInfo in pairs(self.quickslots) do
		local eventTable = quickslotInfo.Quickslot:GetEvents()
		for eventName, _ in pairs(eventTable) do
			if not self.quickslotEvents[eventName] then self.quickslotEvents[eventName] = {} end
			table.insert(self.quickslotEvents[eventName], ID)
		end
	end
end

function SkillBar:FireEvent(eventName)
	local eventInfo = self.eventList[eventName]
	if eventInfo and self.quickslotEvents[eventName] then
		for _, ID in pairs(self.quickslotEvents[eventName]) do
			local quickslot = self.quickslots[ID]
			if quickslot then
				quickslot.Quickslot:HandleEvent(eventInfo)
			end
		end
	end
end

function SkillBar:GetLabelPosition(x, y, currentXPrio)
	if self.alignment == "Left" then
		if self.useStatic then
			return (x - 1) * self.labelSize, (y - 1) * self.labelSize
		else
			return currentXPrio * self.labelSize, (y - 1) * self.labelSize
		end
	elseif self.alignment == "Right" then
		local padding = self.width - self.xCounts[y] * self.labelSize
		if self.useStatic then
			return padding + x * self.labelSize, (y - 1) * self.labelSize
		else
			return padding + currentXPrio * self.labelSize, (y - 1) * self.labelSize
		end
	else
		local padding = self.width / 2 - self.xCounts[y] * (self.labelSize / 2)
		if self.useStatic then
			return padding + x * self.labelSize, (y - 1) * self.labelSize
		else
			return padding + currentXPrio * self.labelSize, (y - 1) * self.labelSize

		end
	end
end

function SkillBar:ForwardEvent(ID)
	local info = self.quickslots[ID]
	if info then
		info.Quickslot:FireEvent(ID)
	else
		ThrowException(string.format("Trying to forward event to non existing ID: %s", tostring(ID)))
	end
end

function SkillBar:SetTimer(ID, timer, maxTimer)
	if not self.quickslots[ID] then
		ThrowException(string.format("Trying to set time for a non existing quickslot ID: %s", tostring(ID)))
		return
	end

	self.quickslots[ID].Quickslot:SetTimer(timer, maxTimer)
end

function SkillBar:Clear()
	for ID, info in pairs(self.quickslots) do
		self.quickslots[ID].Quickslot:Unload()
		self.quickslots[ID] = nil
	end

	self.quickslots = nil
	collectgarbage()
	self.quickslots = {}
end

function SkillBar:Unload()
	self:Clear()
	self:SetParent(nil)
	collectgarbage()
end

function ThrowException(s)
	Turbine.Shell.WriteLine(string.format("%s%s", "<rgb=#FF0000><Prime Exception></rgb> ", tostring(s)))
end