SkillBarQuickslot = class(Turbine.UI.Window)
function SkillBarQuickslot:Constructor(parent, code, layoutTable, eventNames, backpack, callbackFunction)

	--[[
		codeTable:
			Data
			Type
			Item

		layoutTable:
			visibitilityThreshold
			font
			scale
			highlight
			highlightColour
			higlightSize
			keybind text
			circle cooldown
			square cooldown
			locked
			hide while timer active

			duration font
			duration alignment
			show quickslot
			decimal
			decimal threshold
			abbreviation > 60

		eventNames:
			Highlight events
			Opacity events
			Visible events
			Cooldown events

		Quickslot features:
			Highlight (+event)
			Opacity (+event)
			Visible (+event)
			Keybind
			Cooldown (+event)
			Circle cooldown?
			Square cooldown?
			If item detect if empty?
			Cooldown text alignment and font override
			Only show if available?
	]]
    Turbine.UI.Window.Constructor(self)

	self.parent = parent

    self.width = 34
    self.height = 34

	-- layout data
    self.scale = layoutTable.scale or 1
	self.visibitilityThreshold = layoutTable.visibitilityThreshold or 9999
	self.borderColour = layoutTable.borderColour or Turbine.UI.Color(0,0,0)
	self.locked = layoutTable.locked

	self.hideWhileTimerActive = layoutTable.Hide_While_Timer_Active
	self.hideWhileTimerInactive = layoutTable.Hide_While_Timer_Inactive

	self.opacityWhileTimerInactive = layoutTable.Opacity_While_Timer_Inactive or 1
    self.opacityWhileTimerActive = layoutTable.Opacity_While_Timer_Active or 1

	self.highlightOnReady = layoutTable.highlightOnReady or 0

	self.callbackFunction = callbackFunction

	self.highlightColour = layoutTable.HighlightColour or Turbine.UI.Color(0, 0, 0)
	self.higlightSize = layoutTable.higlightSize or 1

	self.keybindFont = layoutTable.Keybind_Font or Turbine.UI.Lotro.Font.FixedSys15
	self.keybind = layoutTable.Show_Keybind
	self.keybindAlignment = layoutTable.Keybind_Alignment
	self.keybind_text = layoutTable.Keybind_Text or ""

	self.durationFont = layoutTable.Duration_Font or Turbine.UI.Lotro.Font.FixedSys15
	self.durationAlignment = layoutTable.Duration_Alignment
	self.showDuration = layoutTable.ShowDuration

	self.decimalThreshold = layoutTable.Decimal_Threshold or 0

	self.abbreviation = layoutTable.Use_Units

	self.hideQuickslot = layoutTable.Hide_Quickslot

	self.borderSize = layoutTable.BorderSize

	-- event data
	self.eventNames = {}

	for keyword in string.gmatch(eventNames, "([^,]+)") do
		self.eventNames[keyword] = true 
	end

	self:SetParent(parent)
    self:SetVisible(not(self.hideWhileTimerActive))
    self:SetBackColor(self.borderColour)
    self:SetOpacity(self.opacityWhileTimerInactive)
	self:SetMouseVisible(false)
    self:SetSize(self.width * self.scale + 2 * self.borderSize, self.height * self.scale + 2 * self.borderSize)


	if not(self.hideQuickslot) then
		self.data = code.Data
		self.type = code.Type
		self.item = code.Item
		if type(code) == "table" and code.Item then
			self.shortcut = Turbine.UI.Lotro.Shortcut(self.type, self.data) 
			if self.shortcut:GetItem() ~= nil and self.shortcut:GetItem():GetItemInfo() then
				self.quickslotName = self.shortcut:GetItem():GetItemInfo():GetName()
				for i = 1, backpack:GetSize(), 1 do
					-- Create the bag slot
					local item = backpack:GetItem(i)
					if item then
						local itemName = item:GetItemInfo():GetName()
						if itemName == self.quickslotName then
							self.QuickslotControl = Turbine.UI.Lotro.ItemControl()
							self.QuickslotControl:SetItem(item)
							self.shortcut = Turbine.UI.Lotro.Shortcut(item)
							self.itemFound = true
							break
						end
					end
				end
			end
		else
			self.success, self.shortcut = pcall(Turbine.UI.Lotro.Shortcut, self.type, self.data)
		end
	end

    self.currentDuration = 0

    self.textFunction = StringifyDuration
	if self.abbreviation then
		self.abbreviationFunction = AbbreviateTime
	end


	if self.scale == 1 then
		self.highlight = ClassSpecificTools.HighlightFrame(self, 1, 1, true)
	else
		self.highlight = ClassSpecificTools.HighlightFrame3(self, 0, 0, (self.width + 2 * self.borderSize) * self.scale, (self.height + 2 * self.borderSize) * self.scale, self.highlightColour, higlightSize)
	end

	if not(self.hideQuickslot) then
		if not(self.itemFound) then
			self.QuickslotControl = Turbine.UI.Lotro.Quickslot()
		end
		self.QuickslotControl:SetParent(self)
		self.QuickslotControl:SetSize(35, 35)
		self.QuickslotControl:SetPosition(-2 + self.borderSize, -2 + self.borderSize)
		if not(self.itemFound) then
			pcall(self.QuickslotControl.SetShortcut, self.QuickslotControl, self.shortcut)
			self.QuickslotControl:SetUseOnRightClick(true)
		end
		if self.hideOnCreation then
			self.QuickslotControl:SetVisible(false)
		end
	end

    self.textLabel = Turbine.UI.Label()
    self.textLabel:SetParent(self)
    self.textLabel:SetTextAlignment(self.durationAlignment or Turbine.UI.ContentAlignment.MiddleCenter)
    self.textLabel:SetFont(self.durationFont or Turbine.UI.Lotro.Font.FixedSys15)
    self.textLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
    self.textLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.textLabel:SetMouseVisible(false)
    self.textLabel:SetMultiline(false)
    self.textLabel:SetSize(self.width * self.scale + 2 * self.borderSize, self.height * self.scale + 2 * self.borderSize)
	self.textLabel:SetVisible(self.showDuration)

	self.keybindLabel = Turbine.UI.Label()
    self.keybindLabel:SetParent(self)
    self.keybindLabel:SetTextAlignment(self.keybindAlignment or Turbine.UI.ContentAlignment.BottomRight)
    self.keybindLabel:SetFont(self.keybindFont or Turbine.UI.Lotro.Font.FixedSys15)
    self.keybindLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
    self.keybindLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.keybindLabel:SetMouseVisible(false)
    self.keybindLabel:SetMultiline(false)
	self.keybindLabel:SetVisible(self.keybind)
	self.keybindLabel:SetText(tostring(self.keybind_text))
	self.keybindLabel:SetPosition(3, 2)
    self.keybindLabel:SetSize((self.width - 6) * self.scale, (self.height - 4) * self.scale)
	self.keybindLabel:SetStretchMode(1)

	if not(self.hideQuickslot) then
		self.QuickslotControl.ShortcutChanged = function()
			if self.locked then
				pcall(self.QuickslotControl.SetShortcut, self.QuickslotControl, self.shortcut)
			end

			self.QuickslotControl:SetStretchMode(1)
			self.QuickslotControl:SetSize(35 * self.scale, 35 * self.scale)
		end

		self.QuickslotControl:SetStretchMode(1)
		self.QuickslotControl:SetSize(35 * self.scale, 35 * self.scale)
	end

	self.currentDuration = 0
    self.maxDuration = 0

	self.ExecuteUpdate = function(delta)
		--Update time
		self.currentDuration = self.currentDuration - delta

		self:SetVisible(self.currentDuration <= self.visibitilityThreshold and not(self.hideWhileTimerActive))

		if self.currentDuration < 0 then
			if self.callback then 
				RemoveCallback(Updater, "ExecuteUpdate", self.callback) 
				self.callback = nil
			end
			self.textLabel:SetStretchMode(nil)
			self:SetOpacity(self.opacityWhileTimerInactive)
			if not(self.hideQuickslot) then self.QuickslotControl:SetOpacity(self.opacityWhileTimerInactive) end
			self.textLabel:SetOpacity(self.opacityWhileTimerInactive)
			self.textLabel:SetStretchMode(1)
			self.textLabel:SetText("")

			if self.active and self.highlightOnReady > 0 then
				self:SetZOrder(100)
				self.highlight:Toggle(true, self.highlightOnReady)
				self:Activate()
				self.active = false
			end

			self:SetVisible(not(self.hideWhileTimerInactive))

			if self.callbackFunction then self.callbackFunction() end
			return true
		end

		-- Update timer label
		self.textLabel:SetStretchMode(nil)
		self.textLabel:SetStretchMode(1)
		if self.currentDuration > 0 and self.currentDuration >= self.decimalThreshold then
			if self.abbreviation and self.currentDuration > 60 then
				self.textLabel:SetText(self.abbreviationFunction(self.currentDuration))
			else
				self.textLabel:SetText(self.textFunction(self.currentDuration))
			end
		else
			self.textLabel:SetText(self.textFunction(self.currentDuration, 1))
		end
	end
end

function SkillBarQuickslot:GetEvents()
	return self.eventNames
end


function SkillBarQuickslot:SetTimer(time, maxTimer)
	local parentparent = self.parent:GetParent()
	if parentparent then parentparent:SetZOrder(100) end
	self.parent:SetZOrder(100)
	self:SetZOrder(100)
    if not(time) then return end
    if time > 0 then
        self.active = true
        if self.highlightOnReady > 0 then
            self.highlight:Toggle(false)
        end
    end

	self.currentDuration = time
	self.maxDuration = maxTimer
	self:SetVisible(self.currentDuration <= self.visibitilityThreshold and not(self.hideWhileTimerActive))
	self:SetOpacity(self.opacityWhileTimerActive)
	if not(self.hideQuickslot) then self.QuickslotControl:SetOpacity(self.opacityWhileTimerActive) end
	self.textLabel:SetOpacity(self.opacityWhileTimerActive)

	if not self.callback then self.callback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
end

function SkillBarQuickslot:HandleEvent(eventInfo)
	if eventInfo then
		if eventInfo.HiglightEvent then
			if self.highlight then
				if eventInfo.HiglightValue then
					self:SetZOrder(100)
				else
					self:SetZOrder(1)
				end
				self.highlight:Toggle(eventInfo.HiglightValue, eventInfo.HiglightTimed and eventInfo.HighlightTimer or nil)
			end
		end

		if eventInfo.OpacityEvent then
			self:SetOpacity(eventInfo.OpacityValue)
			if not self.hideQuickslot then self.QuickslotControl:SetOpacity(eventInfo.OpacityValue) end
			self.textLabel:SetOpacity(eventInfo.OpacityValue)
		end

		if eventInfo.VisibleEvent then
			if eventInfo.VisibilityValueOn then
				self:SetVisible(true)
				self.textLabel:SetVisible(self.showDuration)
				self.keybindLabel:SetVisible(self.keybind)
				if self.QuickslotControl then
					self.QuickslotControl:SetVisible(not self.hideQuickslot)
				end
			end
			if eventInfo.VisibilityValueOff then
				self:SetVisible(false)
				self.textLabel:SetVisible(false)
				self.keybindLabel:SetVisible(false)
				if self.QuickslotControl then
					self.QuickslotControl:SetVisible(false)
				end
				if self.highlight then
					self.highlight:SetVisible(false)
				end
			end
		end 
		
		if eventInfo.TimerEvent then
			self:SetTimer(eventInfo.TimerValue, eventInfo.TimerValue)
		end
	end
end

function SkillBarQuickslot:Unload()
	if self.callback then 
		RemoveCallback(Updater, "ExecuteUpdate", self.callback) 
		self.callback = nil
	end
	--[[
	if self.highlight then
		self.highlight:SetParent(nil)
	end

	self.shortcut = nil
	if not self.hideQuickslot then
		self.QuickslotControl:SetShortcut(nil)
		self.QuickslotControl:SetParent(nil)
	end
--]]
	local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
	self:SetParent(nil)

    --for key, _ in pairs(self) do
    --    self[key] = nil
    --end
    --collectgarbage()
end