TimerLabel = class(Turbine.UI.Window)
function TimerLabel:Constructor(parent, x, y, layoutTable)
    Turbine.UI.Window.Constructor(self)

    self.iconSize = 32
    self.scale = layoutTable.Scale or 1
    self.borderSize = layoutTable.BorderSize or 0
    self.offOpacity = layoutTable.Normal_Opacity
    self.always_Visible = layoutTable.Always_Visible
    self.iconID = layoutTable.Icon

    self.timerFont = layoutTable.Timer_Font or Turbine.UI.Lotro.Font.FixedSys15
    self.textFont = layoutTable.Text_Font or Turbine.UI.Lotro.Font.FixedSys15
    self.fontTimeColour = layoutTable.Timer_Font_Colour or Turbine.UI.Color(1,1,1)
    self.fonttextColour = layoutTable.Text_Font_Colour or Turbine.UI.Color(1,1,1)

    self.borderColour = layoutTable.Border_Colour or Turbine.UI.Color.Black
    self.shade = layoutTable.Shadow_timer
    self.reverse = layoutTable.Increasing_Shadow

    self.Alignment = layoutTable.Alignment
    self.UIWidth = layoutTable.UIWidth
    self.X = layoutTable.X
    self.Y = layoutTable.Y
    self.Advanced = layoutTable.Advanced

    self.Invisible_While_Timer_Active = layoutTable.Invisible_While_Timer_Active
    self.Invisible_While_Timer_Inactive = layoutTable.Invisible_While_Timer_Inactive

    self.durationAlignment = layoutTable.Timer_Alignment
    self.textAlignment = layoutTable.Text_Alignment
    self.duration = 0
    self.maxDuration = 0
    self.scaleStep = 1
    self.scaleChange = false

    self.text = layoutTable.Text

    self.showDuration = layoutTable.Show_Duration

    self.playerName = Turbine.Gameplay.LocalPlayer.GetInstance():GetName()

    self.x = x
    self.y = y

    self:SetParent(parent)
    self:SetSize(self.iconSize + 2 * self.borderSize, self.iconSize + 2 * self.borderSize)
    self:SetVisible(not(self.Invisible_While_Timer_Inactive))
    self:SetOpacity(self.offOpacity)
    self:SetMouseVisible(false)
    self:SetBackColor(self.borderColour)

    self.nameFunction = function(text, name, args)
        return ReplaceTokens(string.gsub(text, "<name>", NameHandlerObject:GetName(name, true, "TimerIcon", "")), args or {})
    end

    self.IconLabel = Turbine.UI.Control()
    self.IconLabel:SetParent(self)
    self.IconLabel:SetPosition(self.borderSize, self.borderSize)
    self.IconLabel:SetBackground(tonumber(self.iconID) or self.iconID)
    self.IconLabel:SetBlendMode(Turbine.UI.BlendMode.Overlay)
    self.IconLabel:SetSize(self.iconSize, self.iconSize)
    self.IconLabel:SetMouseVisible(false)

    self.durationLabel = Turbine.UI.Label()
    self.durationLabel:SetParent(self.IconLabel)
    self.durationLabel:SetSize(self.iconSize, self.iconSize)
    self.durationLabel:SetTextAlignment(self.durationAlignment)
    self.durationLabel:SetFont(self.timerFont)
    self.durationLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
    self.durationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.durationLabel:SetForeColor(self.fontTimeColour)
    self.durationLabel:SetMouseVisible(false)
    self.durationLabel:SetVisible(self.showDuration)

    self.textLabel = Turbine.UI.Label()
    self.textLabel:SetParent(self)
    self.textLabel:SetSize(self.iconSize + 2 * self.borderSize, self.iconSize + 2 * self.borderSize)
    self.textLabel:SetTextAlignment(self.textAlignment)
    self.textLabel:SetFont(self.textFont)
    self.textLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
    self.textLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.textLabel:SetForeColor(self.fonttextColour)
    self.textLabel:SetMouseVisible(false)
    self.textLabel:SetText(self.nameFunction(self.text or "", self.playerName, {}))

    self.currentNumber = -1
    self.currentPercentage = 0




    self.baseImageID = 0x41007E35

    -- Add timer overlay
    --self.highlight = highlight
    --if self.highlight then
    --    self.highlightFrame = HighlightFrame3(self, self.borderSize, self.borderSize, self.normal * self.scale, self.normal * self.scale, highlightColour, highlightSize)
    --    --self.highlightFrame:Toggle(true)
    --end

    if self.shade then
        self.elapsedOverlay = Turbine.UI.Window()
        self.elapsedOverlay:SetStretchMode( 1 )
        self.elapsedOverlay:SetMouseVisible( false )
        --self.elapsedOverlay:SetZOrder( 99 )
        self.elapsedOverlay:SetBackColor( Turbine.UI.Color( 1, 0, 0, 0 ) )
        self.elapsedOverlay:SetBackColorBlendMode( Turbine.UI.BlendMode.Overlay )
        self.elapsedOverlay:SetParent( self )
        self.elapsedOverlay:SetPosition(-1, -1)
        self.elapsedOverlay:SetSize(self.iconSize * self.scale  + 2 * self.borderSize, self.iconSize * self.scale + 2 * self.borderSize)
        self.elapsedOverlay:SetMouseVisible(false)
    end

    self.startTime = Turbine.Engine.GetGameTime()
    self.duration = 0

    self:SetStretchMode(1)
    self:SetSize((self.iconSize + 2 * self.borderSize) * self.scale, (self.iconSize + 2 * self.borderSize) * self.scale)
    self:HandlePosition()
    --self:SetPosition(x - (((self.iconSize + 2 * self.borderSize) * self.scale) - (self.iconSize + 2 * self.borderSize)) / 2, y - (((self.iconSize + 2 * self.borderSize) * self.scale) - (self.iconSize + 2 * self.borderSize)) / 2)

    self.ExecuteUpdate = function(delta)
        self.duration = self.duration - delta
        if self.duration < 0 then 
            if self.wiggle or self.rotate then
                self:SetVisible(true)
                self:SetRotation({x=0,y=0,z=0})
            end    
            self:SetVisible(not(self.Invisible_While_Timer_Inactive))
            if self.durationLabel then
                self.durationLabel:SetStretchMode(nil)
                self.durationLabel:SetStretchMode(1)
                self.durationLabel:SetText("")
            end

            if self.updateCallback then
                RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback)
                self.updateCallback = nil
            end
        end

        if self.scaleChange then
            self.scale = self.scale + self.scaleStep * delta

            if self.scaleDecreasing then
                if self.scale <= self.scaleGoal then
                    self.scaleChange = false
                end
            else
                if self.scale >= self.scaleGoal then
                    self.scaleChange = false
                end
            end

            self:SetStretchMode(nil)
            self:SetSize(self.iconSize + 2 * self.borderSize, self.iconSize + 2 * self.borderSize)
            self:SetStretchMode(1)
            self:SetSize((self.iconSize + 2 * self.borderSize) * self.scale, (self.iconSize + 2 * self.borderSize) * self.scale)
            self:HandlePosition()
        end

        if self.opacityChange then
            local opacity = self:GetOpacity() + self.opacityStep * delta

            if self.opacityDecreasing then
                if opacity <= self.opacityGoal then
                    self.opacityChange = false
                end
            else
                if opacity>= self.opacityGoal then
                    self.opacityChange = false
                end
            end

            self:SetOpacity(opacity)
        end
    
        if self.shade then
            if self.duration > 0 then
                local percentComplete = math.ceil(60 * (self.duration / self.maxDuration))
                if self.reverse then
                    percentComplete = 60 - percentComplete
                end
        
                if self.currentPercentage ~= percentComplete then 
        
                    self.currentPercentage = percentComplete
        
                    local imageId = self.baseImageID + self.currentPercentage            
        
                    self.elapsedOverlay:SetVisible(true)
                    self.elapsedOverlay:SetStretchMode(nil)
                    self.elapsedOverlay:SetSize(32, 32)
                    self.elapsedOverlay:SetBackground(BackgroundHandler:GetBackground(imageId))
                    self.elapsedOverlay:SetStretchMode(1)
                    self.elapsedOverlay:SetSize((self.iconSize + 2 * self.borderSize) + 2 , (self.iconSize + 2 * self.borderSize) + 2 )
                end
            else
                self.elapsedOverlay:SetVisible(false)
            end
        end
    
        if self.rotate and self.duration > 0  then
            self:SetVisible(true)
            local newRotation = self:GetRotation().z + self.rotationPerSecond * delta
            self:SetRotation({x=0,y=0,z=newRotation} )
            if self.shade then
                self.elapsedOverlay:SetRotation({x=0,y=0,z=newRotation})
            end
        end
        
        if self.wiggle and self.duration > 0 then
            self:SetVisible(true)
            local rotation = math.sin(Turbine.Engine.GetGameTime() * 8)*22.5
            self:SetRotation({x=0,y=0,z=rotation})

            if self.shade then
                self.elapsedOverlay:SetRotation({x=0,y=0,z=rotation})
            end
        end
        
        local newNumber = math.round(self.duration)
        if newNumber == self.currentNumber then return end
    
        self.currentNumber = newNumber
    
        if self.durationLabel then
            self.durationLabel:SetStretchMode(nil)
            self.durationLabel:SetStretchMode(1)
            self.durationLabel:SetText(self.currentNumber)
        end

        if self.duration < 0 then 
            self:SetVisible(not(self.Invisible_While_Timer_Inactive))
        end
    end
end

function TimerLabel:HandlePosition()
    local rightOffset = 0
    if self.Advanced then
        self.x = self.X
        self.y = self.Y
    else
        self.x = self:HandleAlignment(self.iconSize + 2 * self.borderSize)
    end
    local x = self.x - (((self.iconSize + 2 * self.borderSize) * self.scale) - (self.iconSize + 2 * self.borderSize)) / 2
    local y = self.y - (((self.iconSize + 2 * self.borderSize) * self.scale) - (self.iconSize + 2 * self.borderSize)) / 2
    self:SetPosition(x, y)
end

function TimerLabel:ToggleVisible(value)
    self:SetVisible(value)
    self.IconLabel:SetVisible(value)
    if self.highlight then
        self.highlightFrame:Toggle(value)
    end
end

function TimerLabel:HandleAlignment(width)
    if self.Alignment == "Middle" then
        return (self.UIWidth - width) / 2
    elseif self.Alignment == "Right" then
        return self.UIWidth - width
    else
        return 0 
    end
end

function TimerLabel:Trigger(triggerInfo, args)
    if triggerInfo.Timer then
        if triggerInfo.Special_Trigger then
            self.duration = args.Duration or 0
            self.maxDuration = args.Duration or 0
        else
            self.duration = triggerInfo.Timer_Duration
            self.maxDuration = triggerInfo.Timer_Duration
        end
        self:SetVisible(not(self.Invisible_While_Timer_Active))
        if not self.updateCallback then
            self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate)
        end
    end

    if triggerInfo.Change_Scale then
        self.scale = triggerInfo.Change_Scale_Number

        self:SetStretchMode(nil)
        self:SetSize(self.iconSize + 2 * self.borderSize, self.iconSize + 2 * self.borderSize)
        self:SetStretchMode(1)
        self:SetSize((self.iconSize + 2 * self.borderSize) * self.scale, (self.iconSize + 2 * self.borderSize) * self.scale)
    end

    if triggerInfo.Change_Scale_Duration then
        self.scaleStep = (triggerInfo.Change_Scale_Number_End - triggerInfo.Change_Scale_Number_Start) / self.duration

        self.scale = triggerInfo.Change_Scale_Number_Start 
        self.scaleChange = true

        self.scaleGoal = triggerInfo.Change_Scale_Number_End 

        self.scaleDecreasing = triggerInfo.Change_Scale_Number_Start > triggerInfo.Change_Scale_Number_End

        self:SetStretchMode(nil)
        self:SetSize(self.iconSize + 2 * self.borderSize, self.iconSize + 2 * self.borderSize)
        self:SetStretchMode(1)
        self:SetSize((self.iconSize + 2 * self.borderSize) * self.scale, (self.iconSize + 2 * self.borderSize) * self.scale)
        self:HandlePosition()
    end

    if triggerInfo.Set_Opacity then
        self:SetOpacity(tonumber(triggerInfo.Set_Opacity_Number) or 1)
    end

    if triggerInfo.Change_Opacity then
        self:SetOpacity(tonumber(triggerInfo.Start_Opacity) or 1)

        self.opacityStep = (triggerInfo.End_Opacity - triggerInfo.Start_Opacity) / self.duration
        self.opacityChange = true
        self.opacityDecreasing = triggerInfo.Start_Opacity > triggerInfo.End_Opacity

        self.opacityGoal = triggerInfo.End_Opacity
    end

    if triggerInfo.Change_Text then
        self.textLabel:SetText(self.nameFunction(tostring(triggerInfo.New_Text) or "", self.playerName, args))
    end

    if triggerInfo.Rotate then
        self.rotationPerSecond = triggerInfo.Rotation_Per_Second
        self.rotate = true
    end

    if triggerInfo.Wiggle then
        self.wiggle = triggerInfo.Start_Wiggle

        if triggerInfo.End_Wiggle then
            self.wiggle = false
            self:SetVisible(true)
            self:SetRotation({x=0,y=0,z=0})
        end
    end
end

function TimerLabel:Unload()
    self:SetParent(nil)
end