GraphWindow = class(Turbine.UI.Window)
function GraphWindow:Constructor(parent, data, x, y, title, width, height, borderWidth, titleHeight, itemWidth, dotWidth, lineWidth, itemColour, dotColour, dotSelectedColour, lineColour)
    Turbine.UI.Window.Constructor(self)

    -- Numerical values
    self.width = width or 800
    self.height = height or 320
    self.outerBorder = borderWidth or 2
    self.scrollbarHeight = 10
    self.titleHeight = titleHeight or 20
    self.listBoxHeight = self.height - 2 * self.outerBorder - self.scrollbarHeight - self.titleHeight
    self.itemWidth = itemWidth or 10
    self.lowestValue = 0
    self.highestValue = 10
    self.dotWidth = dotWidth or 5
    self.lineWidth = lineWidth or 2
    self.itemWidthAdjusted = self.itemWidth - self.dotWidth / 2
    self.itemWidthSquared = math.pow(self.itemWidthAdjusted, 2)
    self.listBoxWidth = self.width - 2 * self.outerBorder
    self.minItem = 0
    self.maxItem = math.floor(self.listBoxWidth / self.itemWidth)
    self.infoWindow = GraphInfoWindow()
    self.data = data or {}
    self.xLabel = "X"
    self.yLabel = "Y"
    self.xUnit = ""
    self.yUnit = ""
    self.xOffset = 0

    -- Colour values
    self.itemColour = itemColour or Turbine.UI.Color(1, 0, 0.5, 0.6)
    self.dotColour = dotColour or Turbine.UI.Color.NavajoWhite
    self.dotSelectedColour = dotSelectedColour or Turbine.UI.Color.DarkRed
    self.lineColour = lineColour or Turbine.UI.Color.White

    if parent then
        self:SetParent(parent)
    end
    self:SetVisible(true)
    self:SetPosition(x or 500, y or 500)
    self:SetSize(self.width, self.height)
    self:SetMouseVisible(false)
    --self:SetBackColor(Turbine.UI.Color.Black)

    self.titleBar = Turbine.UI.Label()
    self.titleBar:SetParent(self)
    self.titleBar:SetVisible(true)
    self.titleBar:SetSize(self.width - 2 * self.outerBorder, self.titleHeight)
    self.titleBar:SetBackColor(Turbine.UI.Color.Black)
    self.titleBar:SetPosition(self.outerBorder, self.outerBorder)
    self.titleBar:SetMouseVisible(false)
    self.titleBar:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.titleBar:SetOutlineColor(Turbine.UI.Color.Black)
    self.titleBar:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.titleBar:SetForeColor(Turbine.UI.Color.White)
    self.titleBar:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.titleBar:SetText(title or "Graph title")
    self.titleBar:SetMouseVisible(false)


    self.graph = Turbine.UI.ListBox()
    self.graph:SetParent(self)
    self.graph:SetSize(self.listBoxWidth, self.listBoxHeight)
    self.graph:SetPosition(self.outerBorder, self.titleHeight + self.outerBorder)
    --self.graph:SetBackColor(self.itemColour)
    self.graph:SetOrientation(Turbine.UI.Orientation.Horizontal)
    self.graph:SetMouseVisible(false)

    self.scrollbar = Turbine.UI.Lotro.ScrollBar()
    self.scrollbar:SetParent(self)
    self.scrollbar:SetPosition(self.outerBorder, self.listBoxHeight + self.outerBorder + self.titleHeight)
    self.scrollbar:SetSize(self.listBoxWidth, self.scrollbarHeight)
    self.scrollbar:SetBackColor(Turbine.UI.Color.Black)
    self.scrollbar:SetVisible(false)
    --self.graph:SetHorizontalScrollBar(scrollbar)

    self:AddPositionItem()

    --[[
    self.data = {
        [1] = {1, 5},
        [2] = {2, 7},
        [3] = {3, 10},
        [4] = {4, 15},
        [5] = {5, 24.9},
        [6] = {6, 25},
        [7] = {7, 15},
        [8] = {8, 10},
        [9] = {9, 7},
        [10] = {10, 5},
    }

    self.data = {}

    for i = 1, 100 do
        self.data[i] = {i, math.random(0, 5000)}
    end
    --]]

    for _, value in pairs(self.data) do if value[2] > self.highestValue then self.highestValue = value[2] end end

    self.dataCount = Tablelength(self.data)
    for key, value in ipairs(self.data) do 
        if key == self.dataCount then break end
        local y1 = (1 - (self.data[key] and self.data[key][2] or 0) / (self.highestValue - self.lowestValue)) * (self.listBoxHeight - self.dotWidth)
        local y2 = (1 - (self.data[key + 1] and self.data[key + 1][2] or 0) / (self.highestValue - self.lowestValue)) * (self.listBoxHeight - self.dotWidth)

        self:ConstructGraphItem(y1, y2, value[1], key, value[2], key)
    end 

    self:GetEndItem()

    self.leftCover = Turbine.UI.Window()
    self.leftCover:SetParent(self)
    self.leftCover:SetVisible(false)
    self.leftCover:SetSize(self.itemWidth, self.listBoxHeight)
    self.leftCover:SetBackColor(Turbine.UI.Color.Black)
    self.leftCover:SetPosition(self.outerBorder, self.outerBorder + self.titleHeight)
    self.leftCover:SetMouseVisible(false)

    self.rightCover = Turbine.UI.Window()
    self.rightCover:SetParent(self)
    self.rightCover:SetVisible(false)
    self.rightCover:SetSize(self.itemWidth, self.listBoxHeight)
    self.rightCover:SetBackColor(Turbine.UI.Color.Black)
    self.rightCover:SetPosition(self.listBoxWidth + self.outerBorder - self.itemWidth, self.outerBorder + self.titleHeight)
    self.rightCover:SetMouseVisible(false)

    self.VisibleChanged = function()
        if self:IsVisible() then
            for i = 1, self.graph:GetItemCount() do
                local item = self.graph:GetItem(i)
                if item.Format then
                    item:Format()
                end
            end
        end
    end
end

function GraphWindow:ConstructGraphItem(dotOneY, dotTwoY, x, ID, showValue, index)
    local listBoxItem = Turbine.UI.Window()
    listBoxItem:SetVisible(true)
    listBoxItem:SetSize(self.itemWidth, self.listBoxHeight)
    --listBoxItem:SetBackColor(self.itemColour)
    listBoxItem:SetMouseVisible(false)
    listBoxItem.ID = ID
    listBoxItem.X = index
    listBoxItem.Value = showValue

    --local dotOneY = (1 - value / (self.highestValue - self.lowestValue)) * (self.listBoxHeight - self.dotWidth)
    --local dotTwoY = (1 - (nextValue or 1) / (self.highestValue - self.lowestValue)) * (self.listBoxHeight - self.dotWidth)

    if math.abs(dotOneY - dotTwoY) <= self.dotWidth then
        listBoxItem.line = Turbine.UI.Window()
        listBoxItem.line:SetParent(listBoxItem)
        listBoxItem.line:SetSize(self.itemWidth, self.lineWidth)
        listBoxItem.line:SetPosition(0, dotOneY)
        listBoxItem.line:SetBackColor(self.lineColour)
        --listBoxItem.line:SetBackColor(Turbine.UI.Color.White)
        listBoxItem.line:SetVisible(true)
        listBoxItem.line:SetMouseVisible(false)

        listBoxItem.PositionChanged = function()
            if self.minItem < listBoxItem.ID and self.maxItem > listBoxItem.ID then
                listBoxItem:SetVisible(true)
            else
                listBoxItem:SetVisible(false)
            end
        end

        listBoxItem.dotOne = Turbine.UI.Control()
        listBoxItem.dotOne:SetParent(listBoxItem)
        listBoxItem.dotOne:SetSize(self.dotWidth, self.dotWidth)
        listBoxItem.dotOne:SetBackColor(self.dotColour)
        listBoxItem.dotOne:SetPosition(0, dotOneY)
        listBoxItem.dotOne:SetMouseVisible(false)
    
        listBoxItem.hoverBox = Turbine.UI.Control()
        listBoxItem.hoverBox:SetParent(listBoxItem)
        listBoxItem.hoverBox:SetSize(20, 20)
        listBoxItem.hoverBox:SetPosition(-5, dotOneY - 5)
    
        listBoxItem.hoverBox.MouseEnter = function()
            self:ShowInfoGraph(listBoxItem.X)
            listBoxItem.dotOne:SetBackColor(self.dotSelectedColour)
        end
    
        listBoxItem.hoverBox.MouseLeave = function()
            self.infoWindow:SetVisible(false)
            listBoxItem.dotOne:SetBackColor(self.dotColour)
        end


        self.graph:AddItem(listBoxItem)
        return 
    end

    local yDiff =  math.pow(dotTwoY - dotOneY, 2)
    local lineLength = math.sqrt(self.itemWidthSquared + yDiff)



    listBoxItem.line = Turbine.UI.Window()
    listBoxItem.line:SetParent(listBoxItem)
    listBoxItem.line:SetSize(lineLength, lineLength)
    listBoxItem.line:SetVisible(true)
    listBoxItem.line:SetMouseVisible(false)

    listBoxItem.lineTexture = Turbine.UI.Control()
    listBoxItem.lineTexture:SetParent(listBoxItem.line)
    listBoxItem.lineTexture:SetSize(lineLength, self.lineWidth)
    listBoxItem.lineTexture:SetBackColor(self.lineColour)
    listBoxItem.lineTexture:SetMouseVisible(false)

    -- [[

    local firstProduct = math.pow(lineLength, 2) + self.itemWidthSquared - yDiff
    local secondProduct = 2 * lineLength * self.itemWidthAdjusted

    listBoxItem.lineAngle = math.deg(math.acos(firstProduct/secondProduct))

    local adjustment = (lineLength / math.sin(math.rad(90))) * math.sin(math.rad(180 - listBoxItem.lineAngle - 90))
    if dotOneY < dotTwoY then
        listBoxItem.line:SetRotation({x=0,y=0,z=-listBoxItem.lineAngle})
        listBoxItem.line:SetPosition(self.itemWidth / 2, dotOneY - adjustment / 2  )
        listBoxItem.lineTexture:SetPosition(0, lineLength - self.lineWidth)
    else
        listBoxItem.line:SetRotation({x=0,y=0,z=listBoxItem.lineAngle})
        listBoxItem.line:SetPosition(self.itemWidth / 2, dotTwoY + adjustment / 2  )
    end

    --]]

    listBoxItem.dotOne = Turbine.UI.Control()
    listBoxItem.dotOne:SetParent(listBoxItem)
    listBoxItem.dotOne:SetSize(self.dotWidth, self.dotWidth)
    listBoxItem.dotOne:SetBackColor(self.dotColour)
    listBoxItem.dotOne:SetPosition(0, dotOneY)
    listBoxItem.dotOne:SetMouseVisible(false)
    listBoxItem.dotOne:SetStretchMode(1)

    listBoxItem.hoverBox = Turbine.UI.Control()
    listBoxItem.hoverBox:SetParent(listBoxItem)
    listBoxItem.hoverBox:SetSize(20, 20)
    listBoxItem.hoverBox:SetPosition(-5, dotOneY - 5)

    listBoxItem.hoverBox.MouseEnter = function()
        self:ShowInfoGraph(listBoxItem.X)
        listBoxItem.dotOne:SetBackColor(self.dotSelectedColour)

    end

    listBoxItem.hoverBox.MouseLeave = function()
        self.infoWindow:SetVisible(false)
        listBoxItem.dotOne:SetBackColor(self.dotColour)
    end

    listBoxItem.Format = function()
        if self.minItem < listBoxItem.ID and self.maxItem > listBoxItem.ID then
            if not(listBoxItem:IsVisible()) then
                listBoxItem:SetVisible(true)
                listBoxItem.line:SetVisible(true)
                listBoxItem.dotOne:SetVisible(true)
                if dotOneY < dotTwoY then
                    listBoxItem.line:SetRotation({x=0,y=0,z=-listBoxItem.lineAngle})
                    listBoxItem.line:SetPosition(self.itemWidth / 2, dotOneY - adjustment / 2  )
                    listBoxItem.lineTexture:SetPosition(0, lineLength - self.lineWidth)
                else
                    listBoxItem.line:SetRotation({x=0,y=0,z=listBoxItem.lineAngle})
                    listBoxItem.line:SetPosition(self.itemWidth / 2, dotTwoY + adjustment / 2  )
                end
            end
        else
            listBoxItem:SetVisible(false)
            listBoxItem.line:SetVisible(false)
            listBoxItem.dotOne:SetVisible(false)
        end
    end

    listBoxItem.PositionChanged = function()
        listBoxItem:Format()
    end

    self.graph:AddItem(listBoxItem)
end

function GraphWindow:ShowInfoGraph(x)
    local xUnit = self.xUnit ~= "" and string.format(" (%s)", self.xUnit) or ""
    local yUnit = self.yUnit ~= "" and string.format(" (%s)", self.yUnit) or ""
    self.infoWindow:View(string.format("%s: %.1f%s\n%s: %.1f%s", self.xLabel, tostring((tonumber(x) or 0) + self.xOffset), xUnit, self.yLabel, self.data[tonumber(x)] and tostring(self.data[tonumber(x)][2]), yUnit))
end

function GraphWindow:SetScrollbar(scrollbar)
    self.scrollbar:SetVisible(false)
    --self.graph:SetHorizontalScrollBar(nil)
    self.graph:SetHorizontalScrollBar(scrollbar)
end

function GraphWindow:SetLabels(x, y)
    self:SetXLabel(x)
    self:SetYLabel(y)
end

function GraphWindow:SetXLabel(x)
    self.xLabel = tostring(x)
end

function GraphWindow:SetYLabel(y)
    self.yLabel = tostring(y)
end

function GraphWindow:SetUnits(x, y)
    self:SetXUnit(x)
    self:SetYUnit(y)
end

function GraphWindow:SetXUnit(x)
    self.xUnit = tostring(x)
end

function GraphWindow:SetYUnit(y)
    self.yUnit = tostring(y)
end

function GraphWindow:SetXOffset(value)
    self.xOffset = value
end

function GraphWindow:AddPositionItem()
    local listBoxItem = Turbine.UI.Window()
    listBoxItem:SetVisible(true)
    listBoxItem:SetSize(self.itemWidth, self.listBoxHeight)
    listBoxItem:SetBackColor(self.itemColour)
    listBoxItem:SetMouseVisible(false)
    listBoxItem:SetVisible(false)
    listBoxItem.PositionChanged = function()
        self.minItem = math.floor(math.abs((listBoxItem:GetLeft() - self.itemWidth)/self.itemWidth)) - 1
        self.maxItem = math.floor(math.abs((listBoxItem:GetLeft() - self.listBoxWidth)/self.itemWidth))
    end
    self.graph:AddItem(listBoxItem)
end

function GraphWindow:GetEndItem()
    local listBoxItem = Turbine.UI.Control()
    listBoxItem:SetVisible(true)
    listBoxItem:SetSize(self.itemWidth, self.listBoxHeight)
    listBoxItem:SetBackColor(self.itemColour)
    listBoxItem:SetMouseVisible(false)
    listBoxItem:SetVisible(true)

    local dotOneY = (1 - (self.data[self.dataCount] and self.data[self.dataCount][2] or 0) / (self.highestValue - self.lowestValue)) * (self.listBoxHeight - self.dotWidth)

    listBoxItem.dotOne = Turbine.UI.Control()
    listBoxItem.dotOne:SetParent(listBoxItem)
    listBoxItem.dotOne:SetSize(self.dotWidth, self.dotWidth)
    listBoxItem.dotOne:SetBackColor(self.dotColour)
    listBoxItem.dotOne:SetPosition(0, dotOneY)
    listBoxItem.dotOne:SetMouseVisible(false)

    listBoxItem.hoverBox = Turbine.UI.Control()
    listBoxItem.hoverBox:SetParent(listBoxItem)
    listBoxItem.hoverBox:SetSize(20, 20)
    listBoxItem.hoverBox:SetPosition(-5, dotOneY - 5)

    listBoxItem.hoverBox.MouseEnter = function()
        self:ShowInfoGraph(self.dataCount)
        listBoxItem.dotOne:SetBackColor(self.dotSelectedColour)
    end

    listBoxItem.hoverBox.MouseLeave = function()
        self.infoWindow:SetVisible(false)
        listBoxItem.dotOne:SetBackColor(self.dotColour)
    end
    

    self.graph:AddItem(listBoxItem)
end

function GraphWindow:Unload()
    for i = 1, self.graph:GetItemCount() do
        local itemList = self.graph:GetItem(1):GetControls()
        for j = 1, itemList:GetCount() do
            itemList:RemoveAt(1)
        end
        self.graph:RemoveItemAt(1)
    end
    self.graph:ClearItems()
    self:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
    collectgarbage()
end

function FormatBigNumberGraph(number)
    if number == nil then return 0 end
    if number < 1000 then
        return string.format("%d", number)
    elseif number < 1e6 then
        return string.format("%.1fk", math.floor(number / 1e3))
    elseif number < 1e9 then
        return string.format("%.1fm", math.floor(number / 1e6))
    elseif number < 1e12 then 
        return string.format("%.1fb", math.floor(number / 1e9))
    else
        return string.format("%.1ft", math.floor(number / 1e12))
    end
end

GraphInfoWindow = class(Turbine.UI.Window)
function GraphInfoWindow:Constructor()
    Turbine.UI.Window.Constructor(self)
    self:SetSize(150, 100)
    self:SetVisible(false)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.TextBox = Turbine.UI.Label()
    self.TextBox:SetParent(self)
    self.TextBox:SetSize(150, 20)
    self.TextBox:SetPosition(3, 3)
    self.TextBox:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    self.TextBox:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
    --self.TextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.TextBox:SetMouseVisible(false)
    self.TextBox:SetBackColor(Turbine.UI.Color(0.8, 0.125, 0.125, 0.125))
end

function GraphInfoWindow:View(Text)
    self:SetVisible(true)
    local x, y = self:GetMousePosition()
    local oldX, oldY = self:GetPosition()
    self:SetPosition((x + oldX) - 75, y + oldY + 5)
    self:SetZOrder(1)
    self.TextBox:SetText(Text or "No info text available.")
    local length = Text and math.ceil(string.len(Text) / 70) + 1 or 1
    self:SetSize(150, length * 22)
    self.TextBox:SetSize(150 - 6, length * 22 - 6)
end